@@ -39,7 +39,14 @@ class CardDescriptor: public MTGCardInstance{
   int init();
   CardDescriptor();
   void unsecureSetTapped(int i);
-	void unsecuresetfresh(int k);
+  void unsecuresetfresh(int k);
+  void setisMultiColored(int w);
+  void setisBlackAndWhite(int w);
+  void setisRedAndBlue(int w);
+  void setisBlackAndGreen(int w);
+  void setisBlueAndGreen(int w);
+  void setisRedAndWhite(int w);
+
   void setNegativeSubtype( string value);
   int counterPower;
   int counterToughness;
@@ -48,6 +55,12 @@ class CardDescriptor: public MTGCardInstance{
   MTGCardInstance * match(MTGCardInstance * card);
   MTGCardInstance * match(MTGGameZone * zone);
   MTGCardInstance * nextmatch(MTGGameZone * zone, MTGCardInstance * previous);
+  
+  int nameComparisonMode;
+  int colorComparisonMode;
+  string compareName;
+  int CDopponentDamaged;
+  int CDcontrollerDamaged;
 };
 
 #endif
