@@ -4,6 +4,7 @@
 #include <JGui.h>
 #include "GuiLayers.h"
 #include "ActionStack.h"
+#include "WResource_Fwd.h"
 
 class GuiLayer;
 class JGuiObject;
@@ -17,24 +18,28 @@ class GameObserver;
 #define DAMAGE_COMBAT 1
 #define DAMAGE_OTHER 2
 
-class Damageable:public Targetable {
- protected:
- public:
-  int life;
-  int poisonCount;
-  int damageCount;
-  int preventable;
-  int type_as_damageable;
-  Damageable(int _life){life=_life;};
+class Damageable:public Targetable
+{
+protected:
+public:
+    int life;
+    int poisonCount;
+    int damageCount;
+    int preventable;
+    int thatmuch;
+    int lifeLostThisTurn;
+    int type_as_damageable;
+    Damageable(int _life){life=_life;lifeLostThisTurn = 0;};
   int getLife(){return life;};
   virtual int dealDamage(int damage){life-=damage;return life;};
   virtual int afterDamage(){return 0;}
   virtual int poisoned(){return 0;}
   virtual int prevented(){return 0;}
-  virtual JQuad * getIcon(){return NULL;};
+  virtual JQuadPtr getIcon(){return JQuadPtr();}
 };
 
-class Damage: public Interruptible {
+class Damage: public Interruptible
+{
  protected:
   void init(MTGCardInstance * source, Damageable * target, int damage, int typeOfDamage);
  public:
@@ -48,8 +53,8 @@ class Damage: public Interruptible {
   virtual ostream& toString(ostream& out) const;
 };
 
-
-class DamageStack : public GuiLayer, public Interruptible{
+class DamageStack : public GuiLayer, public Interruptible
+{
  protected:
   int currentState;
   GameObserver* game;
