@@ -30,44 +30,73 @@ class MTGCardInstance: public CardPrimitive, public MTGCard, public Damageable {
   int nb_damages;
   string sample;
 	int tapped;
-
   int lifeOrig;
   MTGPlayerCards * belongs_to;
   MTGCardInstance * getNextPartner();
   void initMTGCI();
-  int setDefenser(MTGCardInstance * c);
   int addBlocker(MTGCardInstance * c);
   int removeBlocker(MTGCardInstance * c);
   int init();
  public:
 	int setAttacker(int value);
+    int setDefenser(MTGCardInstance * c);
   MTGGameZone * currentZone;
   Pos* view;
   int X;
   int XX;
-  int boughtback;
-  int flashedback;
+  int alternateCostPaid[ManaCost::MANA_PAID_WITH_RETRACE + 1]; 
   int paymenttype;
+  int castMethod; /* Tells if the card reached its current zone by being cast or not (brought into the zone by an effect). non 0 == cast, 0 == not cast */
   int frozen;
   int sunburst;
   int equipment;
+  int auras;
+  bool wasDealtDamage;
+  bool damageToOpponent;
+  bool damageToController;
   int reduxamount;
-	int flanked;
+  int flanked;
   int regenerateTokens;
   int isToken;
-  int stillInUse();
-	int didattacked;
-	int didblocked;
-	int notblocked;
-	int fresh;
-	int MaxLevelUp;
-  Player * lastController;
+  int origpower;
+  int origtoughness;
+  int isMultiColored;
+    int isBlackAndWhite;
+    int isRedAndBlue;
+    int isBlackAndGreen;
+    int isBlueAndGreen;
+    int isRedAndWhite;
+    int isLeveler;
+    bool enchanted;
+    int CDenchanted;
+    int CDdamaged;
+    bool blinked;
+    bool isExtraCostTarget;
+    bool morphed;
+    bool turningOver;
+    bool isMorphed;
+    bool isPhased;
+    int phasedTurn;
+    bool graveEffects;
+    bool exileEffects;
+    bool suspended;
+
+    int stillInUse();
+    int didattacked;
+    int didblocked;
+    int notblocked;
+    int fresh;
+    int MaxLevelUp;
+    Player * lastController;
   MTGGameZone * getCurrentZone();
   MTGGameZone * previousZone;
   MTGCardInstance * previous;
   MTGCardInstance * next;
   int doDamageTest;
   int summoningSickness;
+
+  bool matchesCastFilter(int castMethod);
+
   // The recommended method to test for summoning Sickness !
   int hasSummoningSickness();
   MTGCardInstance * changeController(Player * newcontroller);
@@ -76,6 +105,7 @@ class MTGCardInstance: public CardPrimitive, public MTGCard, public Damageable {
   int typeAsTarget(){return TARGET_CARD;}
   const string getDisplayName() const;
   MTGCardInstance * target;
+  Targetable * backupTargets[MAX_TARGETS];
 
 
   //types
@@ -133,6 +163,11 @@ class MTGCardInstance: public CardPrimitive, public MTGCard, public Damageable {
   int removeProtection(TargetChooser *tc, int erase = 0);
   int protectedAgainst(MTGCardInstance * card);
 
+  vector<TargetChooser *>canttarget;
+  int addCantBeTarget(TargetChooser * tc);
+  int removeCantBeTarget(TargetChooser *tc, int erase = 0);
+  int CantBeTargetby(MTGCardInstance * card);
+  
   vector<TargetChooser *>cantBeBlockedBys;
   int addCantBeBlockedBy(TargetChooser * tc);
   int removeCantBeBlockedBy(TargetChooser *tc, int erase = 0);
@@ -156,7 +191,7 @@ class MTGCardInstance: public CardPrimitive, public MTGCard, public Damageable {
   int isInPlay();
   JSample * getSample();
 
-  JQuad * getIcon();
+  JQuadPtr getIcon();
 
   ostream& toString(ostream&) const;
 
