@@ -11,16 +11,26 @@ The Action Stack contains all information for Game Events that can be interrupte
 #include "GameOptions.h"
 #include "ManaCost.h"
 #include "MTGAbility.h"
+#include "Subtypes.h"
 #include "TargetChooser.h"
 #include "Translate.h"
 #include "WResourceManager.h"
 
 #include <typeinfo>
 
+namespace
+{
+    float kGamepadIconSize = 0.5f;
+
+    std::string kInterruptMessageString("Interrupt?");
+    std::string kInterruptString(": Interrupt");
+    std::string kNoString(": No");
+    std::string kNoToAllString(": No To All");
+}
+
 /*
 NextGamePhase requested by user
 */
-
 int NextGamePhase::resolve()
 {
     GameObserver::GetInstance()->nextGamePhase();
@@ -72,26 +82,33 @@ const string Interruptible::getDisplayName() const
     return typeid(*this).name();
 }
 
+float Interruptible::GetVerticalTextOffset() const
+{
+    static const float kTextVerticalOffset = (mHeight - WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT)->GetHeight()) / 2;
+    return kTextVerticalOffset;
+}
+
 void Interruptible::Render(MTGCardInstance * source, JQuad * targetQuad, string alt1, string alt2, string action,
     bool bigQuad)
 {
     WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);
     mFont->SetColor(ARGB(255,255,255,255));
     mFont->SetScale(DEFAULT_MAIN_FONT_SCALE);
-    mFont->DrawString(_(action).c_str(), x + 30, y, JGETEXT_LEFT);
+
+    mFont->DrawString(_(action).c_str(), x + 35, y + GetVerticalTextOffset(), JGETEXT_LEFT);
     JRenderer * renderer = JRenderer::GetInstance();
-    JQuad * quad = WResourceManager::Instance()->RetrieveCard(source, CACHE_THUMB);
-    if (!quad)
+    JQuadPtr quad = WResourceManager::Instance()->RetrieveCard(source, CACHE_THUMB);
+    if (!quad.get())
         quad = CardGui::AlternateThumbQuad(source);
-    if (quad)
+    if (quad.get())
     {
         quad->SetColor(ARGB(255,255,255,255));
         float scale = mHeight / quad->mHeight;
-        renderer->RenderQuad(quad, x + 10 * scale, y + 15 * scale, 0, scale, scale);
+        renderer->RenderQuad(quad.get(), x + (quad->mWidth * scale / 2), y + (quad->mHeight * scale / 2), 0, scale, scale);
     }
     else if (alt1.size())
     {
-        mFont->DrawString(_(alt1).c_str(), x, y - 15);
+        mFont->DrawString(_(alt1).c_str(), x, y + GetVerticalTextOffset());
     }
 
     if (bigQuad)
@@ -107,12 +124,12 @@ void Interruptible::Render(MTGCardInstance * source, JQuad * targetQuad, string
         targetQuad->SetColor(ARGB(255,255,255,255));
         targetQuad->SetHotSpot(targetQuad->mWidth / 2, targetQuad->mHeight / 2);
         float scale = mHeight / targetQuad->mHeight;
-        renderer->RenderQuad(targetQuad, x + 150, y + 15 * scale, 0, scale, scale);
+        renderer->RenderQuad(targetQuad, x + 150, y + ((mHeight - targetQuad->mHeight) / 2) + targetQuad->mHotSpotY, 0, scale, scale);
         targetQuad->SetHotSpot(backupX, backupY);
     }
     else if (alt2.size())
     {
-        mFont->DrawString(_(alt2).c_str(), x + 120, y);
+        mFont->DrawString(_(alt2).c_str(), x + 120, y + GetVerticalTextOffset());
     }
 }
 
@@ -141,7 +158,7 @@ void StackAbility::Render()
         target = (Damageable *) _target;
     }
 
-    JQuad * quad = NULL;
+    JQuadPtr quad;
     string alt2 = "";
     if (target)
     {
@@ -152,7 +169,7 @@ void StackAbility::Render()
         }
     }
 
-    Interruptible::Render(source, quad, alt1, alt2, action);
+    Interruptible::Render(source, quad.get(), alt1, alt2, action);
 }
 StackAbility::StackAbility(int id, MTGAbility * _ability) :
 Interruptible(id), ability(_ability)
@@ -185,15 +202,37 @@ Interruptible(0)
     cost = NEW ManaCost();
     tc = NULL;
     from = _source->getCurrentZone();
+    payResult = ManaCost::MANA_UNPAID;
+    source->castMethod = Constants::NOT_CAST;
 }
 
 Spell::Spell(int id, MTGCardInstance * _source, TargetChooser * tc, ManaCost * _cost, int payResult) :
 Interruptible(id), tc(tc), cost(_cost), payResult(payResult)
 {
+    if (!cost) cost = NEW ManaCost();
     source = _source;
     mHeight = 40;
     type = ACTION_SPELL;
     from = _source->getCurrentZone();
+    for(int i = 0;i < MAX_TARGETS;i++)
+    {
+    if(tc && tc->targets[i] != NULL)
+    _source->backupTargets[i] = tc->targets[i];
+    }
+
+    // fill information on how the card came into this zone. Right now the quickest way is to do it here, based on how the mana was paid...
+    switch(payResult) {
+        case ManaCost::MANA_UNPAID:
+            source->castMethod = Constants::NOT_CAST;
+            break;
+        case ManaCost::MANA_PAID:
+        case ManaCost::MANA_PAID_WITH_KICKER:
+            source->castMethod = Constants::CAST_NORMALLY;
+            break;
+        default:
+            source->castMethod = Constants::CAST_ALTERNATE;
+            break;
+    }
 }
 
 int Spell::computeX(MTGCardInstance * card)
@@ -251,10 +290,16 @@ Spell::~Spell()
 int Spell::resolve()
 {
     GameObserver * game = GameObserver::GetInstance();
-    if (!source->hasType("instant") && !source->hasType("sorcery"))
+    if (!source->hasType(Subtypes::TYPE_INSTANT) && !source->hasType(Subtypes::TYPE_SORCERY))
     {
         Player * p = source->controller();
+        int castMethod = source->castMethod;
         source = p->game->putInZone(source, from, p->game->battlefield);
+        
+        // We need to get the information about the cast method on both the card in the stack AND the card in play,
+        //so we copy it from the previous card (in the stack) to the new one (in play).
+        source->castMethod = castMethod; 
+
         from = p->game->battlefield;
     }
 
@@ -328,9 +373,9 @@ void Spell::Render()
     string action = source->getName();
     string alt1 = "";
 
-    JQuad * quad = NULL;
     string alt2 = "";
     Damageable * target = getNextDamageableTarget();
+    JQuadPtr quad;
     if (target)
     {
         quad = target->getIcon();
@@ -339,7 +384,7 @@ void Spell::Render()
             alt2 = ((MTGCardInstance *) target)->name;
         }
     }
-    Interruptible::Render(source, quad, alt1, alt2, action, true);
+    Interruptible::Render(source, quad.get(), alt1, alt2, action, true);
 }
 
 ostream& Spell::toString(ostream& out) const
@@ -384,12 +429,12 @@ void PutInGraveyard::Render()
         mFont->DrawString(_("is exiled").c_str(), x + 30, y, JGETEXT_LEFT);
     }
     JRenderer * renderer = JRenderer::GetInstance();
-    JQuad * quad = WResourceManager::Instance()->RetrieveCard(card, CACHE_THUMB);
-    if (quad)
+    JQuadPtr quad = WResourceManager::Instance()->RetrieveCard(card, CACHE_THUMB);
+    if (quad.get())
     {
         quad->SetColor(ARGB(255,255,255,255));
         float scale = 30 / quad->mHeight;
-        renderer->RenderQuad(quad, x, y, 0, scale, scale);
+        renderer->RenderQuad(quad.get(), x, y, 0, scale, scale);
     }
     else
     {
@@ -428,7 +473,7 @@ void DrawAction::Render()
     if (player == GameObserver::GetInstance()->players[1])
         playerId = 2;
     sprintf(buffer, _("Player %i draws %i card").c_str(), playerId, nbcards);
-    mFont->DrawString(buffer, x + 20, y, JGETEXT_LEFT);
+    mFont->DrawString(buffer, x + 35, y + GetVerticalTextOffset(), JGETEXT_LEFT);
 }
 
 ostream& DrawAction::toString(ostream& out) const
@@ -436,7 +481,38 @@ ostream& DrawAction::toString(ostream& out) const
     out << "DrawAction ::: nbcards : " << nbcards << " ; player : " << player;
     return out;
 }
+//////
+LifeAction::LifeAction(int id, Damageable * _target, int amount) :
+Interruptible(id), amount(amount),target(_target)
+{
+}
 
+int LifeAction::resolve()
+{
+target->life += amount;
+    return 1;
+}
+
+void LifeAction::Render()
+{
+    WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::MAIN_FONT);
+    mFont->SetBase(0);
+    mFont->SetScale(DEFAULT_MAIN_FONT_SCALE);
+    char buffer[200];
+    if(amount >= 0)
+        sprintf(buffer, _("Player gains %i life").c_str(), amount);
+    else if(amount >= 0)
+        sprintf(buffer, _("Player loses %i life").c_str(), amount);
+    else
+        sprintf(buffer, _("Nothing happened").c_str(), amount);
+    mFont->DrawString(buffer, x + 20, y, JGETEXT_LEFT);
+}
+
+ostream& LifeAction::toString(ostream& out) const
+{
+    out << "LifeAction ::: amount : " << amount << " ; target : " << target;
+    return out;
+}
 /* The Action Stack itself */
 int ActionStack::addPutInGraveyard(MTGCardInstance * card)
 {
@@ -468,10 +544,19 @@ int ActionStack::addDraw(Player * player, int nb_cards)
     return 1;
 }
 
+int ActionStack::addLife(Damageable * _target, int amount)
+{
+    LifeAction * life = NEW LifeAction(mCount, _target, amount);
+    addAction(life);
+    return 1;
+}
+
 int ActionStack::addDamage(MTGCardInstance * _source, Damageable * _target, int _damage)
 {
     Damage * damage = NEW Damage(_source, _target, _damage);
     addAction(damage);
+    _source->thatmuch = _damage;
+    _target->thatmuch = _damage;
     return 1;
 }
 
@@ -541,8 +626,7 @@ Spell * ActionStack::addSpell(MTGCardInstance * _source, TargetChooser * tc, Man
     }
     Spell * spell = NEW Spell(mCount, _source, tc, mana, payResult);
     addAction(spell);
-    if (!game->players[0]->isAI() && _source->controller() == game->players[0] && 0
-        == options[Options::INTERRUPTMYSPELLS].number)
+    if (!game->players[0]->isAI() && _source->controller() == game->players[0] && 0 == options[Options::INTERRUPTMYSPELLS].number)
         interruptDecision[0] = DONT_INTERRUPT;
     return spell;
 }
@@ -556,8 +640,8 @@ Interruptible * ActionStack::getAt(int id)
     return (Interruptible *) mObjects[id];
 }
 
-ActionStack::ActionStack(GameObserver* game) :
-game(game)
+ActionStack::ActionStack(GameObserver* game)
+    : game(game)
 {
     for (int i = 0; i < 2; i++)
         interruptDecision[i] = 0;
@@ -567,6 +651,19 @@ game(game)
     mode = ACTIONSTACK_STANDARD;
     checked = 0;
 
+    for (int i = 0; i < 8; ++i)
+    {
+        std::ostringstream stream;
+        stream << "iconspsp" << i;
+        pspIcons[i] = WResourceManager::Instance()->RetrieveQuad("iconspsp.png", (float) i * 32, 0, 32, 32, stream.str(), RETRIEVE_MANAGE);
+        pspIcons[i]->SetHotSpot(16, 16);
+    }
+
+    // fix for translation.
+    kInterruptMessageString = _(kInterruptMessageString);
+    kInterruptString = _(kInterruptString);
+    kNoString = _(kNoString);
+    kNoToAllString = _(kNoToAllString);
 }
 
 int ActionStack::has(MTGAbility * ability)
@@ -611,7 +708,6 @@ int ActionStack::resolve()
     }
     if (action->type == ACTION_DAMAGE)
         ((Damage *) action)->target->afterDamage();
-
     if (!getNext(NULL, NOT_RESOLVED))
     {
         for (int i = 0; i < 2; i++)
@@ -776,39 +872,48 @@ void ActionStack::Update(float dt)
         modal = 0;
         if (getLatest(NOT_RESOLVED))
         {
-            int currentPlayerId = 0;
-            int otherPlayerId = 1;
-            if (game->currentlyActing() != game->players[0])
+            Interruptible * currentSpell = (Interruptible *)getLatest(NOT_RESOLVED);
+            MTGCardInstance * card = currentSpell->source;
+            if(card && card->has(Constants::SPLITSECOND))
             {
-                currentPlayerId = 1;
-                otherPlayerId = 0;
-            }
-            if (interruptDecision[currentPlayerId] == NOT_DECIDED)
-            {
-                askIfWishesToInterrupt = game->players[currentPlayerId];
-                game->isInterrupting = game->players[currentPlayerId];
-                modal = 1;
-            }
-            else if (interruptDecision[currentPlayerId] == INTERRUPT)
-            {
-                game->isInterrupting = game->players[currentPlayerId];
-
+                resolve();
             }
             else
             {
-                if (interruptDecision[otherPlayerId] == NOT_DECIDED)
+                int currentPlayerId = 0;
+                int otherPlayerId = 1;
+                if (game->currentlyActing() != game->players[0])
                 {
-                    askIfWishesToInterrupt = game->players[otherPlayerId];
-                    game->isInterrupting = game->players[otherPlayerId];
+                    currentPlayerId = 1;
+                    otherPlayerId = 0;
+                }
+                if (interruptDecision[currentPlayerId] == NOT_DECIDED)
+                {
+                    askIfWishesToInterrupt = game->players[currentPlayerId];
+                    game->isInterrupting = game->players[currentPlayerId];
                     modal = 1;
                 }
-                else if (interruptDecision[otherPlayerId] == INTERRUPT)
+                else if (interruptDecision[currentPlayerId] == INTERRUPT)
                 {
-                    game->isInterrupting = game->players[otherPlayerId];
+                    game->isInterrupting = game->players[currentPlayerId];
+
                 }
                 else
                 {
-                    resolve();
+                    if (interruptDecision[otherPlayerId] == NOT_DECIDED)
+                    {
+                        askIfWishesToInterrupt = game->players[otherPlayerId];
+                        game->isInterrupting = game->players[otherPlayerId];
+                        modal = 1;
+                    }
+                    else if (interruptDecision[otherPlayerId] == INTERRUPT)
+                    {
+                        game->isInterrupting = game->players[otherPlayerId];
+                    }
+                    else
+                    {
+                        resolve();
+                    }
                 }
             }
         }
@@ -830,8 +935,8 @@ void ActionStack::Update(float dt)
 					//this prevents you from "running out of time" while deciding.
 					//before this int was added, it was possible to run out of time if you had 10 stack actions
 					//and set the timer to 4 secs. BUG FIX //http://code.google.com/p/wagic/issues/detail?id=464
-					extraTime = count(NULL,NOT_RESOLVED,NULL);
-					if(extraTime == 0)
+					extraTime = count(0, NOT_RESOLVED, 0);
+					if (extraTime == 0)
 						extraTime = 1;//we never want this int to be 0.
             if (timer < 0)
 							timer = options[Options::INTERRUPT_SECONDS].number * extraTime;
@@ -991,8 +1096,9 @@ void ActionStack::Fizzle(Interruptible * action)
 
 void ActionStack::Render()
 {
-    float x0 = 250;
-    float y0 = 30;
+    static const float kSpacer = 8;
+    static const float x0 = 250;
+    static const float y0 = 0;
     float width = 200;
     float height = 90;
     float currenty = y0 + 5;
@@ -1019,51 +1125,59 @@ void ActionStack::Render()
         //float xScale = width / back->mWidth;
         //float yScale = height / back->mHeight;
         renderer->FillRoundRect(x0 + 16, y0 + 16, width + 2, height + 2, 10, ARGB(128,0,0,0));
-        renderer->FillRoundRect(x0 - 5, y0 - 5, width + 2, height + 2, 10, ARGB(200,0,0,0));
+        renderer->FillRoundRect(x0 - 5, y0, width + 2, height + 2, 10, ARGB(200,0,0,0));
         //renderer->RenderQuad(back,x0,y0,0,xScale, yScale);
-        renderer->DrawRoundRect(x0 - 5, y0 - 5, width + 2, height + 2, 10, ARGB(255,255,255,255));
+        renderer->DrawRoundRect(x0 - 5, y0, width + 2, height + 2, 10, ARGB(255,255,255,255));
+
+        std::ostringstream stream;
+        // WALDORF - changed "interrupt ?" to "Interrupt?". Don't display count down
+        // seconds if the user disables auto progressing interrupts by setting the seconds
+        // value to zero in Options.
+
+        // Mootpoint 01/12/2011: draw the interrupt text first, at the top.  Offset the rest of the 
+        // unresolved stack effects down so that they don't collide with the interrupt text.
+        if (options[Options::INTERRUPT_SECONDS].number == 0)
+            stream << kInterruptMessageString;
+        else
+            stream << kInterruptMessageString << " " << static_cast<int>(timer);
+
+        mFont->DrawString(stream.str(), x0 + 5, currenty);
+
+        static const float kIconVerticalOffset = 24;
+        if (mCount > 1)
+        {
+            renderer->RenderQuad(pspIcons[7].get(), x0 + 10, kIconVerticalOffset, 0, kGamepadIconSize, kGamepadIconSize);
+            mFont->DrawString(kInterruptString, x0 + 19, kIconVerticalOffset - 6);
+
+            renderer->RenderQuad(pspIcons[4].get(), x0 + 97, kIconVerticalOffset, 0, kGamepadIconSize, kGamepadIconSize);
+            mFont->DrawString(kNoString, x0 + 106, kIconVerticalOffset - 6);
+
+            renderer->RenderQuad(pspIcons[6].get(), x0 + 145, kIconVerticalOffset, 0, kGamepadIconSize, kGamepadIconSize);
+            mFont->DrawString(kNoToAllString, x0 + 154, kIconVerticalOffset - 6);
+        }
+        else
+        {
+            renderer->RenderQuad(pspIcons[7].get(), x0 + 40, kIconVerticalOffset, 0, kGamepadIconSize, kGamepadIconSize);
+            mFont->DrawString(kInterruptString, x0 + 49, kIconVerticalOffset - 6);
+
+            renderer->RenderQuad(pspIcons[4].get(), x0 + 140, kIconVerticalOffset - 6, 0, kGamepadIconSize, kGamepadIconSize);
+            mFont->DrawString(kNoString, x0 + 146, kIconVerticalOffset - 6);
+        }
+
+        currenty += kIconVerticalOffset + kSpacer;
 
         for (int i = 0; i < mCount; i++)
         {
             Interruptible * current = (Interruptible *) mObjects[i];
             if (current && current->state == NOT_RESOLVED)
             {
-                current->x = x0 + 5;
-                if (i != mCount - 1)
-                {
-                    current->y = currenty;
-                    currenty += current->mHeight;
-                }
-                else
-                {
-                    current->y = currenty + 40;
-                    currenty += current->mHeight + 40;
-                }
+                current->x = x0;
+                current->y = currenty;
                 current->Render();
+
+                currenty += current->mHeight;
             }
         }
-
-        char buffer[200];
-        // WALDORF - changed "interrupt ?" to "Interrupt?". Don't display count down
-        // seconds if the user disables auto progressing interrupts by setting the seconds
-        // value to zero in Options.
-        if (options[Options::INTERRUPT_SECONDS].number == 0)
-            sprintf(buffer, "%s", _("Interrupt?").c_str());
-        else
-            sprintf(buffer, "%s  %i", _("Interrupt?").c_str(), static_cast<int> (timer));
-
-        //WALDORF - removed all the unnecessary math. just display the prompt at the
-        // top of the box.
-        mFont->DrawString(buffer, x0 + 5, y0);
-
-        if (mCount > 1)
-            sprintf(buffer, "%s", _("X Interrupt - 0 No - [] No to All").c_str());
-        else
-            sprintf(buffer, "%s", _("X Interrupt - 0 No").c_str());
-
-        // WALDORF - puts the button legend right under the prompt. the stack
-        // will be displayed below it now. no more need to do wierd currY math.
-        mFont->DrawString(buffer, x0 + 5, y0 + 14);
     }
     else if (mode == ACTIONSTACK_TARGET && modal)
     {
