@@ -29,6 +29,9 @@ CardPrimitive::CardPrimitive(CardPrimitive * source)
 
     power = source->power;
     toughness = source->toughness;
+    restriction = source->restriction;
+    otherrestriction = source->otherrestriction;
+    suspendedTime = source->suspendedTime;
 
     magicText = source->magicText;
     for (map<string, string>::const_iterator it = source->magicTexts.begin(); it != source->magicTexts.end(); ++it)
@@ -50,6 +53,7 @@ int CardPrimitive::init()
     magicTexts.clear();
     spellTargetType = "";
     alias = 0;
+    hasRestriction = false;
     return 1;
 }
 
@@ -83,6 +87,24 @@ bool CardPrimitive::isSpell()
     return (!isCreature() && !isLand());
 }
 
+void CardPrimitive::setRestrictions(int _restriction)
+{
+    restriction = _restriction;
+}
+int CardPrimitive::getRestrictions()
+{
+    return restriction;
+}
+
+void CardPrimitive::setOtherRestrictions(string _restriction)
+{
+    otherrestriction = _restriction;
+}
+void CardPrimitive::getOtherRestrictions()
+{
+    otherrestriction;
+}
+
 void CardPrimitive::setColor(string _color, int removeAllOthers)
 {
     if (_color.compare("blue") == 0)
@@ -231,12 +253,12 @@ void CardPrimitive::setName(const string& value)
         Subtypes::subtypesList->find(lcname);
 }
 
-const string CardPrimitive::getName() const
+const string& CardPrimitive::getName() const
 {
     return name;
 }
 
-const string CardPrimitive::getLCName() const
+const string& CardPrimitive::getLCName() const
 {
     return lcname;
 }
