@@ -134,6 +134,14 @@ int Counters::removeCounter(const char * _name, int _power, int _toughness)
                 return 0;
             counters[i]->removed();
             counters[i]->nb--;
+            //special case:if a card is suspended and no longer has a time counter when the last is removed, the card is cast.
+            if (target->suspended && !target->counters->hasCounter("time",0,0))
+            {
+                GameObserver * game = game->GetInstance();
+                MTGCardInstance * copy = target->controller()->game->putInZone(target, target->currentZone, target->controller()->game->stack);
+                Spell * spell = game->mLayers->stackLayer()->addSpell(copy, game->targetChooser, NULL,1, 0);
+                game->targetChooser = NULL;
+            }
             return mCount;
         }
     }
