@@ -22,19 +22,19 @@
 #include "GameStateStory.h"
 #include "DeckStats.h"
 #include "DeckMetaData.h"
+#include "DeckManager.h"
 #include "Translate.h"
 #include "WFilter.h"
 
 #define DEFAULT_DURATION .25
 
-MTGAllCards * GameApp::collection = NULL;
 int GameApp::players[] = { 0, 0 };
 int GameApp::HasMusic = 1;
 JMusic * GameApp::music = NULL;
 string GameApp::currentMusicFile = "";
 string GameApp::systemError = "";
 
-JQuad* manaIcons[7] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL };
+JQuadPtr manaIcons[7];
 
 GameState::GameState(GameApp* parent) :
     mParent(parent)
@@ -44,6 +44,9 @@ GameState::GameState(GameApp* parent) :
 
 GameApp::GameApp() :
     JApp()
+#ifdef NETWORK_SUPPORT
+    ,mpNetwork(NULL)
+#endif //NETWORK_SUPPORT
 {
 #ifdef DEBUG
     nbUpdates = 0;
@@ -66,7 +69,6 @@ GameApp::GameApp() :
 
     mCurrentState = NULL;
     mNextState = NULL;
-    collection = NULL;
 
     music = NULL;
 }
@@ -90,7 +92,7 @@ void GameApp::Create()
     LOG("starting Game");
 
     //Find the Res folder
-    std::ifstream mfile("Res.txt");
+    wagic::ifstream mfile("Res.txt");
     string resPath;
     if (mfile)
     {
@@ -122,14 +124,14 @@ void GameApp::Create()
     LOG("Checking for music files");
     //Test for Music files presence
     string filepath = JGE_GET_RES(WResourceManager::Instance()->musicFile("Track0.mp3"));
-    std::ifstream file(filepath.c_str());
+    wagic::ifstream file(filepath.c_str());
     if (file)
         file.close();
     else
         HasMusic = 0;
 
     filepath = JGE_GET_RES(WResourceManager::Instance()->musicFile("Track1.mp3"));
-    std::ifstream file2(filepath.c_str());
+    wagic::ifstream file2(filepath.c_str());
     if (file2)
         file2.close();
     else
@@ -155,13 +157,13 @@ void GameApp::Create()
                     RETRIEVE_MANAGE);
 
     for (int i = sizeof(manaIcons) / sizeof(manaIcons[0]) - 1; i >= 0; --i)
-        if (manaIcons[i])
+        if (manaIcons[i].get())
             manaIcons[i]->SetHotSpot(16, 16);
 
     LOG("--Loading back.jpg");
     WResourceManager::Instance()->RetrieveTexture("back.jpg", RETRIEVE_MANAGE);
-    JQuad * jq = WResourceManager::Instance()->RetrieveQuad("back.jpg", 0, 0, 0, 0, "back", RETRIEVE_MANAGE);
-    if (jq)
+    JQuadPtr jq = WResourceManager::Instance()->RetrieveQuad("back.jpg", 0, 0, 0, 0, "back", RETRIEVE_MANAGE);
+    if (jq.get())
         jq->SetHotSpot(jq->mWidth / 2, jq->mHeight / 2);
 
     WResourceManager::Instance()->RetrieveTexture("back_thumb.jpg", RETRIEVE_MANAGE);
@@ -180,6 +182,8 @@ void GameApp::Create()
     string lang = options[Options::LANG].str;
     std::transform(lang.begin(), lang.end(), lang.begin(), ::tolower);
     WResourceManager::Instance()->InitFonts(lang);
+    Translator::GetInstance()->init();
+    // The translator is ready now.
 
     LOG("--Loading various textures");
     WResourceManager::Instance()->RetrieveTexture("phasebar.png", RETRIEVE_MANAGE);
@@ -191,6 +195,8 @@ void GameApp::Create()
     WResourceManager::Instance()->RetrieveTexture("BattleIcon.png", RETRIEVE_MANAGE);
     WResourceManager::Instance()->RetrieveTexture("DefenderIcon.png", RETRIEVE_MANAGE);
     WResourceManager::Instance()->RetrieveTexture("shadow.png", RETRIEVE_MANAGE);
+    WResourceManager::Instance()->RetrieveTexture("extracostshadow.png", RETRIEVE_MANAGE);
+    WResourceManager::Instance()->RetrieveTexture("morph.jpg", RETRIEVE_MANAGE);
 
     jq = WResourceManager::Instance()->RetrieveQuad("BattleIcon.png", 0, 0, 25, 25, "BattleIcon", RETRIEVE_MANAGE);
     if (jq)
@@ -201,10 +207,16 @@ void GameApp::Create()
     jq = WResourceManager::Instance()->RetrieveQuad("shadow.png", 0, 0, 16, 16, "shadow", RETRIEVE_MANAGE);
     if (jq)
         jq->SetHotSpot(8, 8);
+    jq = WResourceManager::Instance()->RetrieveQuad("extracostshadow.png", 0, 0, 16, 16, "extracostshadow", RETRIEVE_MANAGE);
+    if (jq)
+        jq->SetHotSpot(8, 8);
+    jq = WResourceManager::Instance()->RetrieveQuad("morph.jpg", 0, 0, MTG_MINIIMAGE_WIDTH, MTG_MINIIMAGE_HEIGHT, "morph", RETRIEVE_MANAGE);
+    if (jq)
+        jq->SetHotSpot(static_cast<float> (jq->mTex->mWidth / 2), static_cast<float> (jq->mTex->mHeight / 2));
     jq = WResourceManager::Instance()->RetrieveQuad("phasebar.png", 0, 0, 0, 0, "phasebar", RETRIEVE_MANAGE);
 
     LOG("Init Collection");
-    collection = NEW MTGAllCards();
+    MTGAllCards::loadInstance();
 
     LOG("Creating Game States");
     mGameStates[GAME_STATE_DECK_VIEWER] = NEW GameStateDeckViewer(this);
@@ -260,15 +272,12 @@ void GameApp::Destroy()
         }
     }
 
-    if (collection)
-    {
-        collection->destroyAllCards();
-        SAFE_DELETE(collection);
-    }
-    delete (DeckStats::GetInstance());
+    MTGAllCards::unloadAll();
+
+    DeckManager::EndInstance();
+    DeckStats::EndInstance();
 
     SAFE_DELETE(Subtypes::subtypesList);
-    SAFE_DELETE(DeckMetaDataList::decksMetaData);
 
     playMusic("none");
 
@@ -281,6 +290,10 @@ void GameApp::Destroy()
     options.theGame = NULL;
     LOG("==Destroying GameApp Successful==");
 
+#ifdef TRACK_FILE_USAGE_STATS
+    wagic::ifstream::Dump();
+#endif
+
 }
 
 void GameApp::Update()
