@@ -20,6 +20,10 @@
 #include "TestSuiteAI.h"
 #endif
 
+#ifdef NETWORK_SUPPORT
+#include "NetworkPlayer.h"
+#endif
+
 #if defined (WIN32) || defined (LINUX)
 #include <time.h>
 #endif
@@ -41,6 +45,9 @@ enum ENUM_DUEL_STATE
     DUEL_STATE_PLAY,
     DUEL_STATE_BACK_TO_MAIN_MENU,
     DUEL_STATE_MENU,
+#ifdef NETWORK_SUPPORT
+    DUEL_STATE_OPPONENT_WAIT,
+#endif //NETWORK_SUPPORT
     DUEL_STATE_ERROR
 };
 
@@ -57,11 +64,10 @@ int GameStateDuel::selectedPlayerDeckId = 0;
 int GameStateDuel::selectedAIDeckId = 0;
 
 GameStateDuel::GameStateDuel(GameApp* parent) :
-    GameState(parent)
+GameState(parent)
 {
     for (int i = 0; i < 2; i++)
     {
-        deck[i] = NULL;
         mPlayers[i] = NULL;
     }
     premadeDeck = false;
@@ -78,12 +84,15 @@ GameStateDuel::GameStateDuel(GameApp* parent) :
     credits = NULL;
     rules = NULL;
 
+#ifdef NETWORK_SUPPORT
+    RegisterNetworkPlayers();
+#endif //NETWORK_SUPPORT
+
 }
 
 GameStateDuel::~GameStateDuel()
 {
     End();
-
 }
 
 void GameStateDuel::Start()
@@ -94,7 +103,7 @@ void GameStateDuel::Start()
 
 #ifdef TESTSUITE
     SAFE_DELETE(testSuite);
-    testSuite = NEW TestSuite(JGE_GET_RES("test/_tests.txt").c_str(),mParent->collection);
+    testSuite = NEW TestSuite(JGE_GET_RES("test/_tests.txt").c_str(),MTGCollection());
 #endif
 
     mGamePhase = DUEL_STATE_CHOOSE_DECK1;
@@ -110,16 +119,16 @@ void GameStateDuel::Start()
             decksneeded = 1;
 
             deckmenu = NEW DeckMenu(DUEL_MENU_CHOOSE_DECK, this, Fonts::OPTION_FONT, "Choose a Deck",
-                    GameStateDuel::selectedPlayerDeckId, true);
+                GameStateDuel::selectedPlayerDeckId, true);
             deckmenu->enableDisplayDetailsOverride();
             DeckManager *deckManager = DeckManager::GetInstance();
-            vector<DeckMetaData *> playerDeckList = getValidDeckMetaData(options.profileFile());
+            vector<DeckMetaData *> playerDeckList = BuildDeckList(options.profileFile());
             int nbDecks = playerDeckList.size();
 
             if (nbDecks)
             {
                 decksneeded = 0;
-                if (nbDecks > 1) deckmenu->Add(MENUITEM_RANDOM_PLAYER, "Random", "Play with a random deck.");
+                deckmenu->Add(MENUITEM_RANDOM_PLAYER, "Random", "Play with a random deck.");
             }
 
             renderDeckMenu(deckmenu, playerDeckList);
@@ -145,10 +154,10 @@ void GameStateDuel::Start()
             premadeDeck = true;
             fillDeckMenu(deckmenu, JGE_GET_RES("player/premade"));
         }
-			if (!decksneeded)
-			{
-        deckmenu->Add(MENUITEM_NEW_DECK, "New Deck...", "Create a new deck to play with.");
-			}
+        if (!decksneeded)
+        {
+            deckmenu->Add(MENUITEM_NEW_DECK, "New Deck...", "Create a new deck to play with.");
+        }
         deckmenu->Add(MENUITEM_CANCEL, "Main Menu", "Return to Main Menu");
     }
 
@@ -156,35 +165,43 @@ void GameStateDuel::Start()
     {
         mPlayers[i] = NULL;
     }
-
 }
 
-void GameStateDuel::loadPlayer(int playerId, int decknb, int isAI)
+void GameStateDuel::loadPlayer(int playerId, int decknb, bool isAI, bool isNetwork)
 {
     if (decknb)
     {
         if (!isAI)
         { //Human Player
-            char deckFile[255];
-            if (premadeDeck)
-                sprintf(deckFile, JGE_GET_RES("player/premade/deck%i.txt").c_str(), decknb);
+            if(playerId == 0)
+            {
+                char deckFile[255];
+                if (premadeDeck)
+                    sprintf(deckFile, JGE_GET_RES("player/premade/deck%i.txt").c_str(), decknb);
+                else
+                    sprintf(deckFile, "%s/deck%i.txt", options.profileFile().c_str(), decknb);
+                char deckFileSmall[255];
+                sprintf(deckFileSmall, "player_deck%i", decknb);
+                mPlayers[playerId] = NEW HumanPlayer(deckFile, deckFileSmall);
+#ifdef NETWORK_SUPPORT
+                if(isNetwork)
+                {
+                    ProxyPlayer* mProxy;
+                    mProxy = NEW ProxyPlayer(mPlayers[playerId], mParent->mpNetwork);
+                }
+            }
             else
-                sprintf(deckFile, "%s/deck%i.txt", options.profileFile().c_str(), decknb);
-            char deckFileSmall[255];
-            sprintf(deckFileSmall, "player_deck%i", decknb);
-            MTGDeck * tempDeck = NEW MTGDeck(deckFile, mParent->collection);
-            mPlayers[playerId] = NEW HumanPlayer(tempDeck, deckFile, deckFileSmall);
-
-            deck[playerId] = mPlayers[playerId]->game;
-            delete tempDeck;
+            {   //Remote player
+                mPlayers[playerId] = NEW RemotePlayer(mParent->mpNetwork);
+#endif //NETWORK_SUPPORT
+            }
         }
         else
         { //AI Player, chooses deck
             AIPlayerFactory playerCreator;
             Player * opponent = NULL;
             if (playerId == 1) opponent = mPlayers[0];
-            mPlayers[playerId] = playerCreator.createAIPlayer(mParent->collection, opponent, decknb);
-            deck[playerId] = mPlayers[playerId]->game;
+            mPlayers[playerId] = playerCreator.createAIPlayer(MTGCollection(), opponent, decknb);
         }
     }
     else
@@ -192,8 +209,7 @@ void GameStateDuel::loadPlayer(int playerId, int decknb, int isAI)
         AIPlayerFactory playerCreator;
         Player * opponent = NULL;
         if (playerId == 1) opponent = mPlayers[0];
-        mPlayers[playerId] = playerCreator.createAIPlayer(mParent->collection, opponent);
-        deck[playerId] = mPlayers[playerId]->game;
+        mPlayers[playerId] = playerCreator.createAIPlayer(MTGCollection(), opponent);
     }
 }
 
@@ -212,7 +228,6 @@ void GameStateDuel::loadTestSuitePlayers()
     for (int i = 0; i < 2; i++)
     {
         mPlayers[i] = NEW TestSuiteAI(testSuite, i);
-        deck[i] = mPlayers[i]->game;
     }
     mParent->gameType = testSuite->gameType;
 
@@ -221,7 +236,7 @@ void GameStateDuel::loadTestSuitePlayers()
     game->startGame(rules);
     if (mParent->gameType == GAME_TYPE_MOMIR)
     {
-        game->addObserver(NEW MTGMomirRule(-1, mParent->collection));
+        game->addObserver(NEW MTGMomirRule(-1, MTGCollection()));
     }
 }
 #endif
@@ -231,24 +246,22 @@ void GameStateDuel::End()
     DebugTrace("Ending GameStateDuel");
 
     JRenderer::GetInstance()->EnableVSync(false);
-    DeckManager::EndInstance();
-
-    if (!premadeDeck && mPlayers[0] && mPlayers[1]) // save the stats for the game
+    if (!premadeDeck && mPlayers[0] && mPlayers[1])
+    { // save the stats for the game
         mPlayers[0]->End();
+    }
     else if ( !mPlayers[1] && mPlayers[0] )
         // clean up player object
         SAFE_DELETE( mPlayers[0] );
+    GameObserver::EndInstance(); // this will delete both player objects if a game has been played
 
-    GameObserver::EndInstance(); // this will delete both player objects
     game = NULL;
     premadeDeck = false;
 
     for (int i = 0; i < 2; i++)
     {
         mPlayers[i] = NULL;
-        deck[i] = NULL;
     }
-
     SAFE_DELETE(credits);
     SAFE_DELETE(rules);
 
@@ -266,7 +279,7 @@ void GameStateDuel::End()
 bool GameStateDuel::MusicExist(string FileName)
 {
     string filepath = JGE_GET_RES(WResourceManager::Instance()->musicFile(FileName));
-    std::ifstream file(filepath.c_str());
+    wagic::ifstream file(filepath.c_str());
     if (file)
     {
         file.close();
@@ -278,21 +291,21 @@ bool GameStateDuel::MusicExist(string FileName)
 
 void GameStateDuel::ConstructOpponentMenu()
 {
-	if (opponentMenu == NULL)
-	{
-		opponentMenu = NEW DeckMenu(DUEL_MENU_CHOOSE_OPPONENT, this, Fonts::OPTION_FONT, "Choose Opponent",
-			GameStateDuel::selectedAIDeckId, true);
-		opponentMenu->Add(MENUITEM_RANDOM_AI, "Random");
-		if (options[Options::EVILTWIN_MODE_UNLOCKED].number) opponentMenu->Add(MENUITEM_EVIL_TWIN, "Evil Twin", 
-				_("Can you defeat yourself?").c_str());
-		DeckManager * deckManager = DeckManager::GetInstance();
-		vector<DeckMetaData*> opponentDeckList;
-		int nbUnlockedDecks = options[Options::CHEATMODEAIDECK].number ? 1000 : options[Options::AIDECKS_UNLOCKED].number;
-		opponentDeckList = fillDeckMenu(opponentMenu, JGE_GET_RES("ai/baka"), "ai_baka", mPlayers[0], nbUnlockedDecks);
-		deckManager->updateMetaDataList(&opponentDeckList, true);
-		opponentMenu->Add(MENUITEM_CANCEL, "Cancel", _("Choose a different player deck").c_str());
-		opponentDeckList.clear();
-	}
+    if (opponentMenu == NULL)
+    {
+        opponentMenu = NEW DeckMenu(DUEL_MENU_CHOOSE_OPPONENT, this, Fonts::OPTION_FONT, "Choose Opponent",
+            GameStateDuel::selectedAIDeckId, true);
+        opponentMenu->Add(MENUITEM_RANDOM_AI, "Random");
+        if (options[Options::EVILTWIN_MODE_UNLOCKED].number) opponentMenu->Add(MENUITEM_EVIL_TWIN, "Evil Twin", 
+            _("Can you defeat yourself?").c_str());
+        DeckManager * deckManager = DeckManager::GetInstance();
+        vector<DeckMetaData*> opponentDeckList;
+        int nbUnlockedDecks = options[Options::CHEATMODEAIDECK].number ? 1000 : options[Options::AIDECKS_UNLOCKED].number;
+        opponentDeckList = fillDeckMenu(opponentMenu, JGE_GET_RES("ai/baka"), "ai_baka", mPlayers[0], nbUnlockedDecks);
+        deckManager->updateMetaDataList(&opponentDeckList, true);
+        opponentMenu->Add(MENUITEM_CANCEL, "Cancel", _("Choose a different player deck").c_str());
+        opponentDeckList.clear();
+    }
 }
 
 void GameStateDuel::Update(float dt)
@@ -410,7 +423,7 @@ void GameStateDuel::Update(float dt)
             game->startGame(rules);
             if (mParent->gameType == GAME_TYPE_MOMIR)
             {
-                game->addObserver(NEW MTGMomirRule(-1, mParent->collection));
+                game->addObserver(NEW MTGMomirRule(-1, MTGCollection()));
             }
 
             //start of in game music code
@@ -427,7 +440,7 @@ void GameStateDuel::Update(float dt)
             else if (mParent->gameType == GAME_TYPE_MOMIR)
                 musictrack = "ai_baka_music_momir.mp3";
             else if (mParent->gameType == GAME_TYPE_RANDOM1 || mParent->gameType == GAME_TYPE_RANDOM2) musictrack
-                    = "ai_baka_music_random.mp3";
+                = "ai_baka_music_random.mp3";
 
             if (!MusicExist(musictrack)) musictrack = "ai_baka_music.mp3";
 
@@ -448,29 +461,29 @@ void GameStateDuel::Update(float dt)
                     testSuite->initGame();
                 }
                 else
-                mGamePhase = DUEL_STATE_END;
+                    mGamePhase = DUEL_STATE_END;
             }
             else
 #endif
-            if (mParent->players[0] == PLAYER_TYPE_CPU && mParent->players[1] == PLAYER_TYPE_CPU)
-            {
-                End();
-                Start();
-            }
+                if (mParent->players[0] == PLAYER_TYPE_CPU && mParent->players[1] == PLAYER_TYPE_CPU)
+                {
+                    End();
+                    Start();
+                }
         }
         if (mEngine->GetButtonClick(JGE_BTN_MENU))
         {
             if (!menu)
             {
                 menu = NEW SimpleMenu(DUEL_MENU_GAME_MENU, this, Fonts::MENU_FONT, SCREEN_WIDTH / 2 - 100, 25,
-                        game->players[1]->deckName.c_str());
+                    game->players[1]->deckName.c_str());
                 int cardsinhand = game->players[0]->game->hand->nb_cards;
 
                 //almosthumane - mulligan
                 if ((game->turn < 1) && (cardsinhand != 0) && game->currentGamePhase == Constants::MTG_PHASE_FIRSTMAIN
-                        && game->players[0]->game->inPlay->nb_cards == 0 && game->players[0]->game->graveyard->nb_cards == 0
-                        && game->players[0]->game->exile->nb_cards == 0) //1st Play Check
-                //IF there was no play at the moment automatically mulligan
+                    && game->players[0]->game->inPlay->nb_cards == 0 && game->players[0]->game->graveyard->nb_cards == 0
+                    && game->players[0]->game->exile->nb_cards == 0) //1st Play Check
+                    //IF there was no play at the moment automatically mulligan
                 {
                     menu->Add(MENUITEM_MULLIGAN, "Mulligan");
                 }
@@ -481,6 +494,29 @@ void GameStateDuel::Update(float dt)
             mGamePhase = DUEL_STATE_MENU;
         }
         break;
+#ifdef NETWORK_SUPPORT
+    case DUEL_STATE_OPPONENT_WAIT:
+      {
+        if(mPlayers[1] && mPlayers[1]->game)
+        { // Player loaded
+          menu->Close();
+          SAFE_DELETE(menu);
+          mGamePhase = DUEL_STATE_PLAY;
+        } else if(menu == NULL)
+        {
+          loadPlayer(1, 42/* 0 not good*/, false, true);
+          menu = NEW SimpleMenu(DUEL_STATE_OPPONENT_WAIT, this, Fonts::MENU_FONT, 150, 60);
+          if (menu)
+          {
+              menu->Add(MENUITEM_MAIN_MENU, "Back to main menu");
+          }
+        } else
+        {
+          menu->Update(dt);
+        }
+      }
+      break;
+#endif //NETWORK_SUPPORT
     case DUEL_STATE_MENU:
         menu->Update(dt);
         break;
@@ -498,7 +534,7 @@ void GameStateDuel::Update(float dt)
             menu->Update(dt);
             if (menu->isClosed())
             {
-                PlayerData * playerdata = NEW PlayerData(mParent->collection);
+                PlayerData * playerdata = NEW PlayerData(MTGCollection());
                 playerdata->taskList->passOneDay();
                 playerdata->taskList->save();
                 SAFE_DELETE(playerdata);
@@ -524,58 +560,62 @@ void GameStateDuel::Render()
     switch (mGamePhase)
     {
     case DUEL_STATE_END:
-    {
-        JRenderer * r = JRenderer::GetInstance();
-        r->FillRect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, ARGB(200,0,0,0));
-        credits->Render();
-#ifdef TESTSUITE
-        if (mParent->players[1] == PLAYER_TYPE_TESTSUITE)
         {
-            r->ClearScreen(ARGB(255,0,0,0));
-            char buf[4096];
-            int nbFailed = testSuite->nbFailed;
-            int nbTests = testSuite->nbTests;
-            if (!nbFailed)
-            {
-                sprintf(buf, "All %i tests successful!", nbTests);
-            }
-            else
-            {
-                sprintf(buf, "%i tests out of %i FAILED!", nbFailed, nbTests);
-            }
-            mFont->SetColor(ARGB(255,255,255,255));
-            mFont->DrawString(buf,0,SCREEN_HEIGHT/2);
-            nbFailed = testSuite->nbAIFailed;
-            nbTests = testSuite->nbAITests;
-            if (nbTests)
+            JRenderer * r = JRenderer::GetInstance();
+            r->FillRect(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT, ARGB(200,0,0,0));
+            credits->Render();
+#ifdef TESTSUITE
+            if (mParent->players[1] == PLAYER_TYPE_TESTSUITE)
             {
+                r->ClearScreen(ARGB(255,0,0,0));
+                char buf[4096];
+                int nbFailed = testSuite->nbFailed;
+                int nbTests = testSuite->nbTests;
                 if (!nbFailed)
                 {
-                    sprintf(buf, "AI Tests: All %i tests successful!", nbTests);
+                    sprintf(buf, "All %i tests successful!", nbTests);
                 }
                 else
                 {
-                    sprintf(buf, "AI Tests: %i tests out of %i FAILED!", nbFailed, nbTests);
+                    sprintf(buf, "%i tests out of %i FAILED!", nbFailed, nbTests);
+                }
+                mFont->SetColor(ARGB(255,255,255,255));
+                mFont->DrawString(buf,0,SCREEN_HEIGHT/2);
+                nbFailed = testSuite->nbAIFailed;
+                nbTests = testSuite->nbAITests;
+                if (nbTests)
+                {
+                    if (!nbFailed)
+                    {
+                        sprintf(buf, "AI Tests: All %i tests successful!", nbTests);
+                    }
+                    else
+                    {
+                        sprintf(buf, "AI Tests: %i tests out of %i FAILED!", nbFailed, nbTests);
+                    }
+                    mFont->DrawString(buf,0,SCREEN_HEIGHT/2+20);
                 }
-                mFont->DrawString(buf,0,SCREEN_HEIGHT/2+20);
             }
-        }
 #endif
-        break;
-    }
+            break;
+        }
     case DUEL_STATE_ERROR:
-    {
-        r->ClearScreen(ARGB(200,0,0,0));
-        mFont->DrawString(_("AN ERROR OCCURRED, CHECK FILE NAMES").c_str(), 0, SCREEN_HEIGHT / 2);
-        break;
-    }
+        {
+            r->ClearScreen(ARGB(200,0,0,0));
+            mFont->DrawString(_("AN ERROR OCCURRED, CHECK FILE NAMES").c_str(), 0, SCREEN_HEIGHT / 2);
+            break;
+        }
     case DUEL_STATE_CHOOSE_DECK1:
     case DUEL_STATE_CHOOSE_DECK1_TO_2:
     case DUEL_STATE_CHOOSE_DECK2:
     case DUEL_STATE_CHOOSE_DECK2_TO_PLAY:
     case DUEL_STATE_DECK1_DETAILED_INFO:
     case DUEL_STATE_DECK2_DETAILED_INFO:
-        if (mParent->gameType != GAME_TYPE_CLASSIC)
+        if (mParent->gameType != GAME_TYPE_CLASSIC
+#ifdef NETWORK_SUPPORT
+                && mParent->gameType != GAME_TYPE_SLAVE
+#endif //NETWORK_SUPPORT
+                )
             mFont->DrawString(_("LOADING DECKS").c_str(), 0, SCREEN_HEIGHT / 2);
         else
         {
@@ -592,6 +632,11 @@ void GameStateDuel::Render()
         mFont->DrawString(_("NO DECK AVAILABLE,").c_str(), 0, SCREEN_HEIGHT / 2);
         mFont->DrawString(_("PRESS CIRCLE TO GO TO THE DECK EDITOR!").c_str(), 0, SCREEN_HEIGHT / 2 + 20);
         break;
+#ifdef NETWORK_SUPPORT
+    case DUEL_STATE_OPPONENT_WAIT:
+        if (menu) menu->Render();
+        break;
+#endif //NETWORK_SUPPORT
     case DUEL_STATE_MENU:
     case DUEL_STATE_CANCEL:
     case DUEL_STATE_BACK_TO_MAIN_MENU:
@@ -677,7 +722,7 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
                 if (!popupScreen)
                 {
                     popupScreen = NEW SimplePopup(DUEL_MENU_DETAILED_DECK2_INFO, this, Fonts::MAIN_FONT, "Detailed Information",
-                            selectedDeck, mParent->collection);
+                        selectedDeck, MTGCollection());
                     popupScreen->Render();
                     selectedAIDeckId = selectedDeck->getDeckId();
                 }
@@ -726,7 +771,7 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
             if (!popupScreen)
             {
                 popupScreen = NEW SimplePopup(DUEL_MENU_DETAILED_DECK1_INFO, this, Fonts::MAIN_FONT, "Detailed Information",
-                        selectedDeck, mParent->collection);
+                    selectedDeck, MTGCollection());
                 popupScreen->Render();
                 selectedPlayerDeckId = deckmenu->getSelectedDeckId();
             }
@@ -752,9 +797,22 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
             vector<DeckMetaData *> * playerDeck = deckManager->getPlayerDeckOrderList();
             if (!premadeDeck && controlId > 0) 
                 deckNumber = playerDeck->at(controlId - 1)->getDeckId();
-            loadPlayer(0, deckNumber);
+            loadPlayer(0, deckNumber, false
+#ifdef NETWORK_SUPPORT
+                       ,(mParent->players[1] == PLAYER_TYPE_REMOTE)
+#endif //NETWORK_SUPPORT
+                       );
             deckmenu->Close();
-            mGamePhase = DUEL_STATE_CHOOSE_DECK1_TO_2;
+#ifdef NETWORK_SUPPORT
+            if(mParent->players[1] == PLAYER_TYPE_REMOTE)
+            {   // no need to choose an opponent deck in network mode
+                mGamePhase = DUEL_STATE_OPPONENT_WAIT;
+            }
+            else
+#endif //NETWORK_SUPPORT
+            {
+                mGamePhase = DUEL_STATE_CHOOSE_DECK1_TO_2;
+            }
             playerDeck = NULL;
         }
         else
@@ -780,19 +838,8 @@ void GameStateDuel::ButtonPressed(int controllerId, int controlId)
             break;
         case MENUITEM_MULLIGAN:
             //almosthumane - mulligan
-
-
-            int cardsinhand = game->players[0]->game->hand->nb_cards;
-
-            for (int i = 0; i < cardsinhand; i++) //Discard hand
-                game->currentPlayer->game->putInZone(game->currentPlayer->game->hand->cards[0],
-                        game->currentPlayer->game->hand,
-                        game->currentPlayer->game->library);
-
-            game->currentPlayer->game->library->shuffle(); //Shuffle
-
-            for (int i = 0; i < (cardsinhand - 1); i++)
-                game->draw(); //Draw hand with 1 less card penalty //almhum
+            game->currentPlayer->takeMulligan();
+            
             menu->Close();
             mGamePhase = DUEL_STATE_CANCEL;
             break;
