@@ -26,6 +26,14 @@ float GameStateShop::_y3[] = { 164, 205, 257, 184, 180, 170, 219, 212, 195, 251,
 float GameStateShop::_x4[] = { 76, 90, 65, 131, 171, 221, 123, 187, 225, 141, 237 };
 float GameStateShop::_y4[] = { 169, 188, 250, 182, 182, 168, 220, 208, 198, 259, 245 };
 
+namespace
+{
+    float kGamepadIconSize = 0.5f;
+    std::string kOtherCardsString(": Other cards");
+    std::string kCreditsString("Credits: ");
+}
+
+
 BoosterDisplay::BoosterDisplay(int id, GameObserver* game, int x, int y, JGuiListener * listener, TargetChooser * tc,
                 int nb_displayed_items) :
     CardDisplay(id, game, x, y, listener, tc, nb_displayed_items)
@@ -47,7 +55,6 @@ GameStateShop::GameStateShop(GameApp* parent) :
     menu = NULL;
     for (int i = 0; i < 8; i++)
         altThumb[i] = NULL;
-    mBack = NULL;
     boosterDisplay = NULL;
     taskList = NULL;
     srcCards = NULL;
@@ -67,6 +74,9 @@ GameStateShop::GameStateShop(GameApp* parent) :
         mCounts[i] = 0;
     }
     mTouched = false;
+
+    kOtherCardsString = _(kOtherCardsString);
+    kCreditsString = _(kCreditsString);
 }
 
 GameStateShop::~GameStateShop()
@@ -94,7 +104,7 @@ void GameStateShop::Start()
 
     bigSync = 0;
     shopMenu = NEW WGuiMenu(JGE_BTN_DOWN, JGE_BTN_UP, true, &bigSync);
-    MTGAllCards * ac = GameApp::collection;
+    MTGAllCards * ac = MTGCollection();
     playerdata = NEW PlayerData(ac);
     myCollection = NEW DeckDataWrapper(playerdata->collection);
     pricelist = NEW PriceList(JGE_GET_RES("settings/prices.dat").c_str(), ac);
@@ -132,7 +142,13 @@ void GameStateShop::Start()
     altThumb[6] = WResourceManager::Instance()->RetrieveTexture("land_thumb.jpg", RETRIEVE_LOCK);
     altThumb[7] = WResourceManager::Instance()->RetrieveTexture("gold_thumb.jpg", RETRIEVE_LOCK);
 
-    mBack = WResourceManager::Instance()->GetQuad("back");
+    for (int i = 0; i < 8; ++i)
+    {
+        std::ostringstream stream;
+        stream << "iconspsp" << i;
+        pspIcons[i] = WResourceManager::Instance()->RetrieveQuad("iconspsp.png", (float) i * 32, 0, 32, 32, stream.str(), RETRIEVE_MANAGE);
+        pspIcons[i]->SetHotSpot(16, 16);
+    }
 
     JRenderer::GetInstance()->EnableVSync(true);
 
@@ -294,7 +310,7 @@ void GameStateShop::purchaseBooster(int controlId)
     mInventory[controlId]--;
     SAFE_DELETE(booster);
     deleteDisplay();
-    booster = NEW MTGDeck(mParent->collection);
+    booster = NEW MTGDeck(MTGCollection());
     boosterDisplay = NEW BoosterDisplay(12, NULL, SCREEN_WIDTH - 200, SCREEN_HEIGHT / 2, this, NULL, 5);
     mBooster[controlId].addToDeck(booster, srcCards);
 
@@ -356,9 +372,6 @@ void GameStateShop::updateCounts()
 }
 void GameStateShop::load()
 {
-    int nbsets = 0;
-    int nbboostersets = 0;
-
     for (int i = 0; i < BOOSTER_SLOTS; i++)
     {
         mBooster[i].randomize(packlist);
@@ -658,17 +671,17 @@ void GameStateShop::Render()
     if (mStage == STAGE_FADE_IN)
         return;
 
-    JQuad * mBg = WResourceManager::Instance()->RetrieveTempQuad("shop.jpg", TEXTURE_SUB_5551);
-    if (mBg)
-        r->RenderQuad(mBg, 0, 0);
+    JQuadPtr mBg = WResourceManager::Instance()->RetrieveTempQuad("shop.jpg", TEXTURE_SUB_5551);
+    if (mBg.get())
+        r->RenderQuad(mBg.get(), 0, 0);
 
-    JQuad * quad = WResourceManager::Instance()->RetrieveTempQuad("shop_light.jpg", TEXTURE_SUB_5551);
-    if (quad)
+    JQuadPtr quad = WResourceManager::Instance()->RetrieveTempQuad("shop_light.jpg", TEXTURE_SUB_5551);
+    if (quad.get())
     {
         r->EnableTextureFilter(false);
         r->SetTexBlend(BLEND_SRC_ALPHA, BLEND_ONE);
         quad->SetColor(ARGB(lightAlpha,255,255,255));
-        r->RenderQuad(quad, 0, 0);
+        r->RenderQuad(quad.get(), 0, 0);
         r->SetTexBlend(BLEND_SRC_ALPHA, BLEND_ONE_MINUS_SRC_ALPHA);
         r->EnableTextureFilter(true);
     }
@@ -718,13 +731,14 @@ void GameStateShop::Render()
 
     //Render the info bar
     r->FillRect(0, SCREEN_HEIGHT - 17, SCREEN_WIDTH, 17, ARGB(128,0,0,0));
-    char c[512];
-    sprintf(c, _("credits: %i").c_str(), playerdata->credits);
+    std::ostringstream stream;
+    stream << kCreditsString << playerdata->credits;
     mFont->SetColor(ARGB(255,255,255,255));
-    mFont->DrawString(c, 5, SCREEN_HEIGHT - 12);
-    sprintf(c, "%s", _("[]:other cards").c_str());
-    float len = 4 + mFont->GetStringWidth(c);
-    mFont->DrawString(c, SCREEN_WIDTH - len, SCREEN_HEIGHT - 14);
+    mFont->DrawString(stream.str(), 5, SCREEN_HEIGHT - 14);
+
+    float len = 4 + mFont->GetStringWidth(kOtherCardsString.c_str());
+	r->RenderQuad(pspIcons[6].get(), SCREEN_WIDTH - len - kGamepadIconSize - 10, SCREEN_HEIGHT - 8, 0, kGamepadIconSize, kGamepadIconSize);
+    mFont->DrawString(kOtherCardsString, SCREEN_WIDTH - len, SCREEN_HEIGHT - 14);
 
     mFont->SetColor(ARGB(255,255,255,0));
     mFont->DrawString(descPurchase(bigSync.getPos()).c_str(), SCREEN_WIDTH / 2, SCREEN_HEIGHT - 14, JGETEXT_CENTER);
@@ -875,7 +889,7 @@ void ShopBooster::randomCustom(MTGPacks * packlist)
 }
 void ShopBooster::randomStandard()
 {
-    int mSet = -1;
+
     MTGSetInfo * si = setlist.randomSet(-1);
     mainSet = si;
     altSet = NULL;
@@ -936,7 +950,7 @@ void ShopBooster::addToDeck(MTGDeck * d, WSrcCards * srcCards)
 bool ShopBooster::unitTest()
 {
     //this tests the default random pack creation.
-    MTGDeck * d = NEW MTGDeck(GameApp::collection);
+    MTGDeck * d = NEW MTGDeck(MTGCollection());
     char result[1024];
 
     randomStandard();
