@@ -5,6 +5,7 @@
 #include "GameOptions.h"
 #include "WEvent.h"
 #include "MTGDeck.h"
+#include "Subtypes.h"
 
 #if defined (WIN32) || defined (LINUX)
 #include <time.h>
@@ -14,13 +15,18 @@
 //Players Game
 //------------------------------
 
+MTGPlayerCards::MTGPlayerCards()
+{
+    init();
+}
+
 MTGPlayerCards::MTGPlayerCards(int * idList, int idListSize)
 {
     init();
     int i;
     for (i = 0; i < idListSize; i++)
     {
-        MTGCard * card = GameApp::collection->getCardById(idList[i]);
+        MTGCard * card = MTGCollection()->getCardById(idList[i]);
         if (card)
         {
             MTGCardInstance * newCard = NEW MTGCardInstance(card, this);
@@ -63,10 +69,28 @@ MTGPlayerCards::~MTGPlayerCards()
     SAFE_DELETE(removedFromGame);
     SAFE_DELETE(garbage);
     SAFE_DELETE(temp);
+    SAFE_DELETE(playRestrictions);
+}
+
+void MTGPlayerCards::beforeBeginPhase()
+{
+    delete garbage;
+    garbage = NEW MTGGameZone();
+    garbage->setOwner(this->owner);
+
+    library->beforeBeginPhase();
+    graveyard->beforeBeginPhase();
+    hand->beforeBeginPhase();
+    inPlay->beforeBeginPhase();
+    stack->beforeBeginPhase();
+    removedFromGame->beforeBeginPhase();
+    garbage->beforeBeginPhase();
+    temp->beforeBeginPhase();
 }
 
 void MTGPlayerCards::setOwner(Player * player)
 {
+    this->owner = player;
     library->setOwner(player);
     graveyard->setOwner(player);
     hand->setOwner(player);
@@ -102,7 +126,7 @@ void MTGPlayerCards::OptimizedHand(Player * who,int amount, int lands, int creat
         Player * p = who;
         MTGCardInstance * card = NULL;
         MTGGameZone * z = p->game->library;
-        MTGGameZone * e = p->game->temp;
+
         int optimizedland = 0;
         int optimizedothercards = 0;
         int optimizedcreatures = 0;
@@ -110,7 +134,7 @@ void MTGPlayerCards::OptimizedHand(Player * who,int amount, int lands, int creat
         {
             MTGCardInstance * _card = z->cards[j];
             //-------------
-            if (_card->hasType("Land") && optimizedland < lands)
+            if (_card->isLand() && optimizedland < lands)
             {
                 card = _card;
                 if (card)
@@ -120,8 +144,8 @@ void MTGPlayerCards::OptimizedHand(Player * who,int amount, int lands, int creat
                 }
             }
             //----------------first try to optimize a few cards that cost 2 or less.
-            if (_card->getManaCost()->getConvertedCost() <= 2 && optimizedothercards < othercards && !_card->hasType("Land")
-                            && !_card->hasType("Creature"))
+            if (_card->getManaCost()->getConvertedCost() <= 2 && optimizedothercards < othercards && !_card->isLand()
+                            && !_card->isCreature())
             {
                 card = _card;
                 if (card)
@@ -130,7 +154,7 @@ void MTGPlayerCards::OptimizedHand(Player * who,int amount, int lands, int creat
                     optimizedothercards += 1;
                 }
             }
-            if (_card->getManaCost()->getConvertedCost() <= 2 && optimizedcreatures < creatures && _card->hasType("Creature"))
+            if (_card->getManaCost()->getConvertedCost() <= 2 && optimizedcreatures < creatures && _card->isCreature())
             {
                 card = _card;
                 if (card)
@@ -145,8 +169,8 @@ void MTGPlayerCards::OptimizedHand(Player * who,int amount, int lands, int creat
         {
             MTGCardInstance * _card = z->cards[k];
 
-            if (_card->getManaCost()->getConvertedCost() <= 3 && optimizedothercards < othercards && (!_card->hasType("Land")
-                            || _card->hasType("Creature")))
+            if (_card->getManaCost()->getConvertedCost() <= 3 && optimizedothercards < othercards && (!_card->isLand()
+                            || _card->isCreature()))
             {
                 card = _card;
                 if (card)
@@ -155,8 +179,8 @@ void MTGPlayerCards::OptimizedHand(Player * who,int amount, int lands, int creat
                     optimizedothercards += 1;
                 }
             }
-            if (_card->getManaCost()->getConvertedCost() <= 3 && optimizedcreatures < creatures && (_card->hasType("Creature")
-                            || !_card->hasType("Land")))
+            if (_card->getManaCost()->getConvertedCost() <= 3 && optimizedcreatures < creatures && (_card->isCreature()
+                            || !_card->isLand()))
             {
                 card = _card;
                 if (card)
@@ -236,6 +260,8 @@ void MTGPlayerCards::init()
     exile = removedFromGame;
     garbage = NEW MTGGameZone();
     temp = NEW MTGGameZone();
+
+    playRestrictions = NEW PlayRestrictions();
 }
 
 void MTGPlayerCards::showHand()
@@ -243,95 +269,33 @@ void MTGPlayerCards::showHand()
     hand->debugPrint();
 }
 
+// Moves a card to its owner's graveyard
 MTGCardInstance * MTGPlayerCards::putInGraveyard(MTGCardInstance * card)
 {
-    MTGCardInstance * copy = NULL;
-    MTGGraveyard * grave = card->owner->game->graveyard;
-    if (inPlay->hasCard(card))
-    {
-        copy = putInZone(card, inPlay, grave);
-    }
-    else if (stack->hasCard(card))
-    {
-        copy = putInZone(card, stack, grave);
-    }
-    else
-    {
-        copy = putInZone(card, hand, grave);
-    }
-    return copy;
-
+    return putInZone(card, card->currentZone, card->owner->game->graveyard);
 }
 
+// Moves a card to its owner's exile
 MTGCardInstance * MTGPlayerCards::putInExile(MTGCardInstance * card)
 {
-    MTGCardInstance * copy = NULL;
-    MTGRemovedFromGame * exile = card->owner->game->exile;
-    if (inPlay->hasCard(card))
-    {
-        copy = putInZone(card, inPlay, exile);
-    }
-    else if (stack->hasCard(card))
-    {
-        copy = putInZone(card, stack, exile);
-    }
-    else if (graveyard->hasCard(card))
-    {
-        copy = putInZone(card, graveyard, exile);
-    }
-    else
-    {
-        copy = putInZone(card, hand, exile);
-    }
-    return copy;
+    return putInZone(card, card->currentZone, card->owner->game->exile);
 }
+
+// Moves a card to its owner's library
 MTGCardInstance * MTGPlayerCards::putInLibrary(MTGCardInstance * card)
 {
-    MTGCardInstance * copy = NULL;
-    MTGLibrary * library = card->owner->game->library;
-    MTGHand * hand = card->owner->game->hand;
-    if (inPlay->hasCard(card))
-    {
-        copy = putInZone(card, inPlay, library);
-    }
-    else if (stack->hasCard(card))
-    {
-        copy = putInZone(card, stack, library);
-    }
-    else if (graveyard->hasCard(card))
-    {
-        copy = putInZone(card, graveyard, library);
-    }
-    else
-    {
-        copy = putInZone(card, hand, library);
-    }
-    return copy;
+    return putInZone(card, card->currentZone, card->owner->game->library);
 }
 
+// Moves a card to its *owner's* (not controller!) hand
 MTGCardInstance * MTGPlayerCards::putInHand(MTGCardInstance * card)
 {
-    MTGCardInstance * copy = NULL;
-    MTGHand * hand = card->owner->game->hand;
-    if (inPlay->hasCard(card))
-    {
-        copy = putInZone(card, inPlay, hand);
-    }
-    else if (stack->hasCard(card))
-    {
-        copy = putInZone(card, stack, hand);
-    }
-    else if (graveyard->hasCard(card))
-    {
-        copy = putInZone(card, graveyard, hand);
-    }
-    else
-    {
-        copy = putInZone(card, hand, hand);
-    }
-    return copy;
+    return putInZone(card, card->currentZone, card->owner->game->hand);
 }
 
+
+// Moves a card from one zone to another
+// If the card is not actually in the expected "from" zone, does nothing and returns null 
 MTGCardInstance * MTGPlayerCards::putInZone(MTGCardInstance * card, MTGGameZone * from, MTGGameZone * to)
 {
     MTGCardInstance * copy = NULL;
@@ -347,58 +311,58 @@ MTGCardInstance * MTGPlayerCards::putInZone(MTGCardInstance * card, MTGGameZone
         doCopy = 0;
     }
 
-    if ((copy = from->removeCard(card, doCopy)))
+    if (!(copy = from->removeCard(card, doCopy)))
+        return NULL; //ERROR
+
+    if (options[Options::SFXVOLUME].number > 0)
     {
-        if (options[Options::SFXVOLUME].number > 0)
+        if (to == g->players[0]->game->graveyard || to == g->players[1]->game->graveyard)
         {
-            if (to == g->players[0]->game->graveyard || to == g->players[1]->game->graveyard)
+            if (card->isCreature())
             {
-                if (card->isCreature())
-                {
-                    JSample * sample = WResourceManager::Instance()->RetrieveSample("graveyard.wav");
-                    if (sample)
-                        JSoundSystem::GetInstance()->PlaySample(sample);
-                }
+                JSample * sample = WResourceManager::Instance()->RetrieveSample("graveyard.wav");
+                if (sample)
+                    JSoundSystem::GetInstance()->PlaySample(sample);
             }
         }
-
-        MTGCardInstance * ret = copy;
-
-        to->addCard(copy);
-
-        //The "Temp" zone are purely for code purposes, and we don't want the abilities engine to
-        //Trigger when cards move in this zone
-        // Additionally, when they mve "from" this zone,
-        // we trick the engine into believing that they moved from the zone the card was previously in
-        // See http://code.google.com/p/wagic/issues/detail?id=335
-        {
-            if (to == g->players[0]->game->temp || to == g->players[1]->game->temp)
-            {
-                //don't send event when moving to temp
-                return ret;
-            }
-
-            if (from == g->players[0]->game->temp || from == g->players[1]->game->temp)
-            {
-                //remove temporary stuff
-                MTGCardInstance * previous = copy->previous;
-                MTGCardInstance * previous2 = previous->previous;
-                from = previous->previousZone;
-                copy->previous = previous2;
-                if (previous2)
-                    previous2->next = copy;
-                previous->previous = NULL;
-                previous->next = NULL;
-                SAFE_DELETE(previous);
-            }
-        }
-
-        WEvent * e = NEW WEventZoneChange(copy, from, to);
-        g->receiveEvent(e);
-
-        return ret;
     }
-    return card; //Error
+
+    MTGCardInstance * ret = copy;
+
+    to->addCard(copy);
+
+    //The "Temp" zone are purely for code purposes, and we don't want the abilities engine to
+    //Trigger when cards move in this zone
+    // Additionally, when they move "from" this zone,
+    // we trick the engine into believing that they moved from the zone the card was previously in
+    // See http://code.google.com/p/wagic/issues/detail?id=335
+    {
+        if (to == g->players[0]->game->temp || to == g->players[1]->game->temp)
+        {
+            //don't send event when moving to temp
+            return ret;
+        }
+
+        if (from == g->players[0]->game->temp || from == g->players[1]->game->temp)
+        {
+            //remove temporary stuff
+            MTGCardInstance * previous = copy->previous;
+            MTGCardInstance * previous2 = previous->previous;
+            from = previous->previousZone;
+            copy->previous = previous2;
+            if (previous2)
+                previous2->next = copy;
+            previous->previous = NULL;
+            previous->next = NULL;
+            SAFE_DELETE(previous);
+        }
+    }
+
+    WEvent * e = NEW WEventZoneChange(copy, from, to);
+    g->receiveEvent(e);
+
+    return ret;
+
 }
 
 void MTGPlayerCards::discardRandom(MTGGameZone * from, MTGCardInstance * source)
@@ -420,7 +384,14 @@ int MTGPlayerCards::isInPlay(MTGCardInstance * card)
     }
     return 0;
 }
-
+int MTGPlayerCards::isInZone(MTGCardInstance * card,MTGGameZone * zone)
+{
+    if (zone->hasCard(card))
+    {
+        return 1;
+    }
+    return 0;
+}
 //--------------------------------------
 // Zones specific code
 //--------------------------------------
@@ -441,6 +412,11 @@ MTGGameZone::~MTGGameZone()
     owner = NULL;
 }
 
+void MTGGameZone::beforeBeginPhase()
+{
+    cardsSeenThisTurn.clear();
+};
+
 void MTGGameZone::setOwner(Player * player)
 {
     for (int i = 0; i < nb_cards; i++)
@@ -474,6 +450,7 @@ MTGCardInstance * MTGGameZone::removeCard(MTGCardInstance * card, int createCopy
                 copy->view = card->view;
                 copy->isToken = card->isToken;
                 copy->X = card->X;
+                copy->XX = card->X/2;
 
                 //stupid bug with tokens...
                 if (card->model == card)
@@ -502,17 +479,31 @@ MTGCardInstance * MTGGameZone::hasCard(MTGCardInstance * card)
 int MTGGameZone::countByType(const char * value)
 {
     int result = 0;
+    int subTypeId = Subtypes::subtypesList->find(value);
     for (int i = 0; i < (nb_cards); i++)
     {
-        if (cards[i]->hasType(value))
+        if (cards[i]->hasType(subTypeId))
         {
             result++;
         }
     }
     return result;
-
 }
 
+int MTGGameZone::countByCanTarget(TargetChooser * tc)
+{
+if(!tc)
+return 0;
+    int result = 0;
+    for (int i = 0; i < (nb_cards); i++)
+    {
+        if (tc->canTarget(cards[i]))
+        {
+            result++;
+        }
+    }
+    return result;
+}
 MTGCardInstance * MTGGameZone::findByName(string name)
 {
     for (int i = 0; i < (nb_cards); i++)
@@ -537,6 +528,54 @@ int MTGGameZone::hasType(const char * value)
     return 0;
 }
 
+int MTGGameZone::hasPrimaryType(const char * value,const char * secondvalue)
+{
+    for (int i = 0; i < (nb_cards); i++)
+    {
+        if (cards[i]->hasType(value) && cards[i]->hasType(secondvalue))
+        {
+            return 1;
+        }
+    }
+    return 0;
+}
+
+int MTGGameZone::hasSpecificType(const char * value,const char * secondvalue)
+{
+    for (int i = 0; i < (nb_cards); i++)
+    {
+        if (cards[i]->hasType(value) && cards[i]->hasSubtype(secondvalue))
+        {
+            return 1;
+        }
+    }
+    return 0;
+}
+
+int MTGGameZone::hasTypeButNotType(const char * value,const char * secondvalue)
+{
+    for (int i = 0; i < (nb_cards); i++)
+    {
+        if (cards[i]->hasType(value) && cards[i]->hasSubtype(value) && !cards[i]->hasType(secondvalue) && !cards[i]->hasSubtype(secondvalue))
+        {
+            return 1;
+        }
+    }
+    return 0;
+}
+
+int MTGGameZone::hasName(string value)
+{
+    for (int i = 0; i < (nb_cards); i++)
+    {
+        if (cards[i]->name == value)
+        {
+            return 1;
+        }
+    }
+    return 0;
+}
+
 int MTGGameZone::hasColor(int value)
 {
     for (int i = 0; i < (nb_cards); i++)
@@ -573,6 +612,32 @@ int MTGGameZone::hasAbility(int ability)
     return 0;
 }
 
+int MTGGameZone::seenThisTurn(TargetChooser * tc, int castMethod)
+{
+    //The following 2 lines modify the passed TargetChooser. Call this function with care :/
+    tc->setAllZones(); // This is to allow targetting cards without caring about the actual zone
+    tc->targetter = NULL;
+
+    int count = 0;
+    for (vector<MTGCardInstance *>::iterator iter = cardsSeenThisTurn.begin(); iter != cardsSeenThisTurn.end(); ++iter)
+    {
+        MTGCardInstance * c = (*iter);
+        if (c->matchesCastFilter(castMethod) &&  tc->canTarget(c))
+            count++;
+    }
+    return count;
+}
+
+int MTGGameZone::seenThisTurn(string targetChooserDefinition, int castMethod)
+{
+    TargetChooserFactory tcf;
+    TargetChooser *tc = tcf.createTargetChooser(targetChooserDefinition, NULL);
+    int result = seenThisTurn(tc, castMethod);
+    delete(tc);
+    return result;
+}
+
+
 void MTGGameZone::cleanupPhase()
 {
     for (int i = 0; i < (nb_cards); i++)
@@ -581,14 +646,7 @@ void MTGGameZone::cleanupPhase()
 
 void MTGGameZone::shuffle()
 {
-    int i;
-    for (i = 0; i < (nb_cards); i++)
-    {
-        int r = i + (WRand() % (nb_cards - i)); // Random remaining position.
-        MTGCardInstance * temp = cards[i];
-        cards[i] = cards[r];
-        cards[r] = temp;
-    }
+    std::random_shuffle(cards.begin(), cards.end());
 }
 
 void MTGGameZone::addCard(MTGCardInstance * card)
@@ -596,6 +654,7 @@ void MTGGameZone::addCard(MTGCardInstance * card)
     if (!card)
         return;
     cards.push_back(card);
+    cardsSeenThisTurn.push_back(card);
     nb_cards++;
     cardsMap[card] = 1;
     card->lastController = this->owner;
@@ -683,6 +742,75 @@ void MTGLibrary::shuffleTopToBottom(int nbcards)
     }
 }
 
+MTGGameZone * MTGGameZone::intToZone(int zoneId, Player * p, Player * p2)
+{
+
+    switch (zoneId)
+    {
+    case MY_GRAVEYARD:
+        return p->game->graveyard;
+    case OPPONENT_GRAVEYARD:
+        return p->opponent()->game->graveyard;
+
+    case MY_BATTLEFIELD:
+        return p->game->inPlay;
+    case OPPONENT_BATTLEFIELD:
+        return p->opponent()->game->inPlay;
+    case BATTLEFIELD:
+        return p->game->inPlay;
+
+    case MY_HAND:
+        return p->game->hand;
+    case OPPONENT_HAND:
+        return p->opponent()->game->hand;
+
+    case MY_EXILE:
+        return p->game->removedFromGame;
+    case OPPONENT_EXILE:
+        return p->opponent()->game->removedFromGame;
+
+
+    case MY_LIBRARY:
+        return p->game->library;
+    case OPPONENT_LIBRARY:
+        return p->opponent()->game->library;
+    case LIBRARY:
+        return p->game->library;
+
+    case MY_STACK:
+        return p->game->stack;
+    case OPPONENT_STACK:
+        return p->opponent()->game->stack;
+    case STACK:
+        return p->game->stack;
+    }
+
+    if (!p2) return NULL;
+    switch (zoneId)
+    {    
+    case TARGET_CONTROLLER_GRAVEYARD:
+        return p2->game->graveyard;
+
+    case TARGET_CONTROLLER_BATTLEFIELD:
+        return p2->game->inPlay;
+
+    case TARGET_CONTROLLER_HAND:
+        return p2->game->hand;
+
+    case TARGET_CONTROLLER_EXILE:
+        return p2->game->removedFromGame;
+
+    case TARGET_CONTROLLER_LIBRARY:
+        return p2->game->library;
+
+    case TARGET_CONTROLLER_STACK:
+        return p2->game->stack;
+
+    default:
+        return NULL;
+    }
+}
+
 MTGGameZone * MTGGameZone::intToZone(int zoneId, MTGCardInstance * source, MTGCardInstance * target)
 {
     Player *p, *p2;
@@ -693,94 +821,68 @@ MTGGameZone * MTGGameZone::intToZone(int zoneId, MTGCardInstance * source, MTGCa
         p = source->controller();
     if (!target)
     {
-        p2 = p;
-        target = source;//hack ?
+        if(source->target)
+        {
+        //bug case, this is a patchwork fix for now
+        //we need to find the root cause of why the 2nd variable is not returning the target.
+            p2 = source->target->controller();
+            target = source->target;
+        }
+        else
+        {
+        //bug or bug case default to 
+            p2 = source->controller();
+                target = source;
+        }
     }
     else
         p2 = target->controller();
 
+
+    MTGGameZone * result = intToZone(zoneId, p, p2);
+    if (result) return result;
+
     switch (zoneId)
     {
-    case MY_GRAVEYARD:
-        return p->game->graveyard;
-    case OPPONENT_GRAVEYARD:
-        return p->opponent()->game->graveyard;
     case TARGET_OWNER_GRAVEYARD:
         return target->owner->game->graveyard;
-    case TARGET_CONTROLLER_GRAVEYARD:
-        return p2->game->graveyard;
     case GRAVEYARD:
         return target->owner->game->graveyard;
     case OWNER_GRAVEYARD:
         return target->owner->game->graveyard;
 
-    case MY_BATTLEFIELD:
-        return p->game->inPlay;
-    case OPPONENT_BATTLEFIELD:
-        return p->opponent()->game->inPlay;
     case TARGET_OWNER_BATTLEFIELD:
         return target->owner->game->inPlay;
-    case TARGET_CONTROLLER_BATTLEFIELD:
-        return p2->game->inPlay;
-    case BATTLEFIELD:
-        return p->game->inPlay;
     case OWNER_BATTLEFIELD:
         return target->owner->game->inPlay;
 
-    case MY_HAND:
-        return p->game->hand;
-    case OPPONENT_HAND:
-        return p->opponent()->game->hand;
     case TARGET_OWNER_HAND:
         return target->owner->game->hand;
-    case TARGET_CONTROLLER_HAND:
-        return p2->game->hand;
     case HAND:
         return target->owner->game->hand;
     case OWNER_HAND:
         return target->owner->game->hand;
 
-    case MY_EXILE:
-        return p->game->removedFromGame;
-    case OPPONENT_EXILE:
-        return p->opponent()->game->removedFromGame;
     case TARGET_OWNER_EXILE:
         return target->owner->game->removedFromGame;
-    case TARGET_CONTROLLER_EXILE:
-        return p2->game->removedFromGame;
     case EXILE:
         return target->owner->game->removedFromGame;
     case OWNER_EXILE:
         return target->owner->game->removedFromGame;
 
-    case MY_LIBRARY:
-        return p->game->library;
-    case OPPONENT_LIBRARY:
-        return p->opponent()->game->library;
     case TARGET_OWNER_LIBRARY:
         return target->owner->game->library;
-    case TARGET_CONTROLLER_LIBRARY:
-        return p2->game->library;
-    case LIBRARY:
-        return p->game->library;
     case OWNER_LIBRARY:
         return target->owner->game->library;
 
-    case MY_STACK:
-        return p->game->stack;
-    case OPPONENT_STACK:
-        return p->opponent()->game->stack;
     case TARGET_OWNER_STACK:
         return target->owner->game->stack;
-    case TARGET_CONTROLLER_STACK:
-        return p2->game->stack;
-    case STACK:
-        return p->game->stack;
     case OWNER_STACK:
         return target->owner->game->stack;
     default:
         return NULL;
     }
+ 
 }
 
 int MTGGameZone::zoneStringToId(string zoneName)
@@ -848,29 +950,63 @@ ostream& MTGGameZone::toString(ostream& out) const
 }
 ostream& MTGLibrary::toString(ostream& out) const
 {
-    return out << "Library " << *owner;
+    return out << "Library " << owner->getDisplayName();
 }
 ostream& MTGGraveyard::toString(ostream& out) const
 {
-    return out << "Graveyard " << *owner;
+    return out << "Graveyard " << owner->getDisplayName();
 }
 ostream& MTGHand::toString(ostream& out) const
 {
-    return out << "Hand " << *owner;
+    return out << "Hand " << owner->getDisplayName();
 }
 ostream& MTGRemovedFromGame::toString(ostream& out) const
 {
-    return out << "RemovedFromGame " << *owner;
+    return out << "RemovedFromGame " << owner->getDisplayName();
 }
 ostream& MTGStack::toString(ostream& out) const
 {
-    return out << "Stack " << *owner;
+    return out << "Stack " << owner->getDisplayName();
 }
 ostream& MTGInPlay::toString(ostream& out) const
 {
-    return out << "InPlay " << *owner;
+    return out << "InPlay " << owner->getDisplayName();
 }
 ostream& operator<<(ostream& out, const MTGGameZone& z)
 {
     return z.toString(out);
 }
+ostream& operator<<(ostream& out, const MTGPlayerCards& z)
+{
+    out << z.library->nb_cards << " ";
+    for (int i = 0; i < z.library->nb_cards; i++)
+        out << z.library->cards[i]->getMTGId() << " ";
+
+    return out;
+}
+
+istream& operator>>(istream& in, MTGPlayerCards& z)
+{
+    int nb, mtgid;
+    in >> nb;
+
+    for (int i = 0; i < z.library->nb_cards; i++)
+    {
+        SAFE_DELETE( z.library->cards[i] );
+    }
+    z.library->cards.clear();
+    z.library->cardsMap.clear();
+
+    for (int i = 0; i < nb; i++)
+    {
+        in >> mtgid;
+        MTGCard * card = MTGCollection()->getCardById(mtgid);
+        if (card)
+        {
+            MTGCardInstance * newCard = NEW MTGCardInstance(card, &z);
+            z.library->addCard(newCard);
+        }
+    }
+
+    return in;
+}
