@@ -28,11 +28,7 @@ int MTGPackEntryRandom::addCard(WSrcCards *pool, MTGDeck *to)
 }
 int MTGPackEntrySpecific::addCard(WSrcCards *pool, MTGDeck *to)
 {
-    int fails = 0;
-    //Ignores pool entirely.
-    MTGAllCards * ac = GameApp::collection;
-
-    if (!card)
+	if (!card)
         return copies;
     for (int i = 0; i < copies; i++)
         to->add(card);
@@ -75,7 +71,7 @@ WSrcCards * MTGPack::getPool(string poolstr)
         s = poolstr.find("unlocked");
         string sub = poolstr;
         if (s != string::npos)
-            sub = poolstr.substr(s + 8);
+            sub = trim(poolstr.substr(s + 8));
         if (sub.size())
         {
             mySrc->addFilter(ff->Construct(sub));
@@ -89,11 +85,11 @@ WSrcCards * MTGPack::getPool(string poolstr)
         if (sub.size())
         {
             mySrc->addFilter(ff->Construct(sub));
-            mySrc->loadMatches(GameApp::collection);
+            mySrc->loadMatches(MTGCollection());
             mySrc->bakeFilters();
         }
         else
-            mySrc->loadMatches(GameApp::collection);
+            mySrc->loadMatches(MTGCollection());
     }
     mySrc->Shuffle();
     return mySrc;
@@ -227,7 +223,7 @@ void MTGPack::load(string filename)
                     es->copies = atoi(holder);
                 else
                     es->copies = 1;
-                es->card = GameApp::collection->getCardByName(pEntry->Value());
+                es->card = MTGCollection()->getCardByName(pEntry->Value());
                 s->addEntry(es);
             }
             else if (tag == "random_card")
