@@ -8,6 +8,7 @@
 
 #include "MTGGameZones.h"
 #include "MTGAbility.h"
+#include "AllAbilities.h"
 #include "DeckManager.h"
 #include "AIPlayer.h"
 #include <JLogger.h>
@@ -17,7 +18,7 @@ int Rules::getMTGId(string cardName)
     int cardnb = atoi(cardName.c_str());
     if (cardnb) return cardnb;
     if (cardName.compare("*") == 0) return -1; //Any card
-    MTGCard * card = GameApp::collection->getCardByName(cardName);
+    MTGCard * card = MTGCollection()->getCardByName(cardName);
     if (card) return card->getMTGId();
     DebugTrace("RULES: Can't find card:" << cardName.c_str());
     return 0;
@@ -104,12 +105,12 @@ void RulesState::parsePlayerState(int playerId, string s)
             playerData[playerId].life = atoi((s.substr(limiter + 1)).c_str());
             return;
         }
-        else if (areaS.compare("poisonCount") == 0)
+        else if (areaS.compare("poisoncount") == 0)
         {
             playerData[playerId].poisonCount = atoi((s.substr(limiter + 1)).c_str());
             return;
         }
-        else if (areaS.compare("damageCount") == 0)
+        else if (areaS.compare("damagecount") == 0)
         {
             playerData[playerId].damageCount = atoi((s.substr(limiter + 1)).c_str());
             return;
@@ -130,6 +131,11 @@ void RulesState::parsePlayerState(int playerId, string s)
             playerData[playerId].manapool = ManaCost::parseManaCost(s.substr(limiter + 1));
             return;
         }
+        else if (areaS.compare("customphasering") == 0)
+        {
+            playerData[playerId].phaseRing = s.substr(limiter + 1);
+            return;
+        }
         else if (areaS.compare("auto") == 0)
         {
             playerData[playerId].extraRules.push_back(s.substr(limiter + 1));
@@ -180,12 +186,16 @@ void Rules::addExtraRules()
             MTGPlayerCards * hand = NULL;
             int handsize = 7;
             int difficultyRating = 0;
-						int Optimizedhandcheat = options[Options::OPTIMIZE_HAND].number;
-
+            int Optimizedhandcheat = options[Options::OPTIMIZE_HAND].number;
             MTGAbility * a = af.parseMagicLine(initState.playerData[i].extraRules[j], id++, NULL, &MTGCardInstance::ExtraRules[i]);
             if (p->playMode != Player::MODE_TEST_SUITE && g->mRules->gamemode != GAME_TYPE_MOMIR && g->mRules->gamemode
-                            != GAME_TYPE_RANDOM1 && g->mRules->gamemode != GAME_TYPE_RANDOM2 && g->mRules->gamemode
-                            != GAME_TYPE_STORY && (!g->players[0] == PLAYER_TYPE_CPU && !g->players[1] == PLAYER_TYPE_CPU))//keep this out of mimor and other game modes.
+                != GAME_TYPE_RANDOM1 && g->mRules->gamemode != GAME_TYPE_RANDOM2 && g->mRules->gamemode
+                != GAME_TYPE_STORY && 
+                g->mRules->gamemode != GAME_TYPE_DEMO && (!g->players[0] == PLAYER_TYPE_CPU && !g->players[1] == PLAYER_TYPE_CPU)
+#ifdef NETWORK_SUPPORT
+                && !g->players[1] == PLAYER_TYPE_REMOTE
+#endif //NETWORK_SUPPORT
+                    )//keep this out of mimor and other game modes.
             {
                 difficultyRating = DeckManager::getDifficultyRating(g->players[0], g->players[1]);
             }
@@ -194,31 +204,31 @@ void Rules::addExtraRules()
             {
                 if (a->oneShot)
                 {
-									if (( p->isAI() ||( !p->isAI() && Optimizedhandcheat)) && a->aType == MTGAbility::STANDARD_DRAW && difficultyRating == EASY && p->playMode
-                                    != Player::MODE_TEST_SUITE && g->mRules->gamemode != GAME_TYPE_MOMIR && g->mRules->gamemode
-                                    != GAME_TYPE_RANDOM1 && g->mRules->gamemode != GAME_TYPE_RANDOM2 && g->mRules->gamemode
-                                    != GAME_TYPE_STORY)//stupid protections to keep this out of mimor and other game modes.
+                    if (((p->isAI() && p->playMode
+                        != Player::MODE_AI && p->opponent()->playMode
+                        != Player::MODE_AI)||( !p->isAI() && Optimizedhandcheat)) && a->aType == MTGAbility::STANDARD_DRAW &&
+                        difficultyRating != HARD && p->playMode
+                        != Player::MODE_TEST_SUITE && g->mRules->gamemode != GAME_TYPE_MOMIR && g->mRules->gamemode
+                        != GAME_TYPE_RANDOM1 && g->mRules->gamemode != GAME_TYPE_RANDOM2 && g->mRules->gamemode
+                        != GAME_TYPE_STORY)//stupid protections to keep this out of mimor and other game modes.
                     {
-                        handsize = a->nbcardAmount;
-                        ((AIPlayer *) p)->forceBestAbilityUse = true;
-                        ((AIPlayer *) p)->agressivity += 100;
-                        hand->OptimizedHand(p,handsize, 3, 1, 3);//easy decks get a major boost, open hand is 2lands,1 creature under 3 mana,3spells under 3 mana.
+                        handsize = ((AADrawer *)a)->getNumCards();
+                        if(difficultyRating == EASY)
+                        {
+                            ((AIPlayer *) p)->forceBestAbilityUse = true;
+                            ((AIPlayer *) p)->agressivity += 100;
+                            hand->OptimizedHand(p,handsize, 3, 1, 3);//easy decks get a major boost, open hand is 2lands,1 creature under 3 mana,3spells under 3 mana.
+                        }
+                        else if (difficultyRating == NORMAL)
+                        {
+                            hand->OptimizedHand(p,handsize, 1, 0, 2);//give the Ai deck a tiny boost by giving it 1 land and 2 spells under 3 manacost.
+                        }
+                        else
+                        {
+                            hand->OptimizedHand(p,handsize, 3, 1, 3);//no rating fall out case.
+                        }
                     }
-                    else if (( p->isAI() ||( !p->isAI() && Optimizedhandcheat)) && a->aType == MTGAbility::STANDARD_DRAW && difficultyRating == NORMAL && p->playMode
-                                    != Player::MODE_TEST_SUITE && g->mRules->gamemode != GAME_TYPE_MOMIR && g->mRules->gamemode
-                                    != GAME_TYPE_RANDOM1 && g->mRules->gamemode != GAME_TYPE_RANDOM2 && g->mRules->gamemode
-                                    != GAME_TYPE_STORY)//stupid protections to keep this out of mimor and other game modes.
-                    {
-                        handsize = a->nbcardAmount; 
-                        hand->OptimizedHand(p,handsize, 1, 0, 2);//give the Ai deck a tiny boost by giving it 1 land and 2 spells under 3 manacost.
-                    }else if (( !p->isAI() && Optimizedhandcheat) && a->aType == MTGAbility::STANDARD_DRAW && p->playMode
-                                    != Player::MODE_TEST_SUITE && g->mRules->gamemode != GAME_TYPE_MOMIR && g->mRules->gamemode
-                                    != GAME_TYPE_RANDOM1 && g->mRules->gamemode != GAME_TYPE_RANDOM2 && g->mRules->gamemode
-                                    != GAME_TYPE_STORY)
-										{
-									      hand->OptimizedHand(p,handsize, 3, 1, 3);
-										}
-										else
+                    else
                     {//resolve normally if the deck is listed as hard.
                         a->resolve();
                     }
@@ -258,7 +268,7 @@ Player * Rules::loadPlayerMomir(int isAI)
     string deckFileSmall = "momir";
     char empty[] = "";
 
-    MTGDeck * tempDeck = NEW MTGDeck(GameApp::collection); //Autogenerate a momir deck. Leave the "momir.txt" bits below for stats.
+    MTGDeck * tempDeck = NEW MTGDeck(MTGCollection()); //Autogenerate a momir deck. Leave the "momir.txt" bits below for stats.
     tempDeck->addRandomCards(12, 0, 0, Constants::RARITY_L, "Forest");
     tempDeck->addRandomCards(12, 0, 0, Constants::RARITY_L, "Plains");
     tempDeck->addRandomCards(12, 0, 0, Constants::RARITY_L, "Swamp");
@@ -267,9 +277,9 @@ Player * Rules::loadPlayerMomir(int isAI)
 
     Player *player = NULL;
     if (!isAI) // Human Player
-        player = NEW HumanPlayer(tempDeck, options.profileFile("momir.txt", "", true).c_str(), deckFileSmall);
+        player = NEW HumanPlayer(options.profileFile("momir.txt", "", true).c_str(), deckFileSmall, tempDeck);
     else
-        player = NEW AIMomirPlayer(tempDeck, options.profileFile("momir.txt", "", true).c_str(), deckFileSmall, empty);
+        player = NEW AIMomirPlayer(options.profileFile("momir.txt", "", true).c_str(), deckFileSmall, empty, tempDeck);
 
     delete tempDeck;
     return player;
@@ -286,7 +296,7 @@ Player * Rules::loadPlayerRandom(int isAI, int mode)
 
     string lands[] = { "forest", "forest", "island", "mountain", "swamp", "plains", "forest" };
 
-    MTGDeck * tempDeck = NEW MTGDeck(GameApp::collection);
+    MTGDeck * tempDeck = NEW MTGDeck(MTGCollection());
     tempDeck->addRandomCards(9, 0, 0, -1, lands[color1].c_str());
     tempDeck->addRandomCards(9, 0, 0, -1, lands[color2].c_str());
     tempDeck->addRandomCards(1, 0, 0, 'U', "land");
@@ -302,9 +312,9 @@ Player * Rules::loadPlayerRandom(int isAI, int mode)
 
     Player *player = NULL;
     if (!isAI) // Human Player
-        player = NEW HumanPlayer(tempDeck, deckFile, deckFileSmall);
+        player = NEW HumanPlayer(deckFile, deckFileSmall, tempDeck);
     else
-        player = NEW AIPlayerBaka(tempDeck, deckFile, deckFileSmall, "");
+        player = NEW AIPlayerBaka(deckFile, deckFileSmall, "", tempDeck);
 
     delete tempDeck;
     return player;
@@ -330,13 +340,14 @@ Player * Rules::initPlayer(int playerId)
             return loadPlayerRandom(isAI, GAME_TYPE_RANDOM2);
         }
     }
+    p->phaseRing = initState.playerData[playerId].phaseRing;
     return p;
 }
 
 MTGDeck * Rules::buildDeck(int playerId)
 {
     int nbcards = 0;
-    MTGDeck * deck = NEW MTGDeck(GameApp::collection);
+    MTGDeck * deck = NEW MTGDeck(MTGCollection());
     for (int j = 0; j < 4; j++)
     {
         for (size_t k = 0; k < initState.playerData[playerId].zones[j].cards.size(); k++)
@@ -378,6 +389,16 @@ void Rules::initGame()
     DebugTrace("RULES Init Game\n");
 
     //Set the current player/phase
+    if (g->currentPlayer->playMode
+        != Player::MODE_TEST_SUITE && /*g->mRules->gamemode != GAME_TYPE_MOMIR && g->mRules->gamemode
+        != GAME_TYPE_RANDOM1 && g->mRules->gamemode != GAME_TYPE_RANDOM2 &&*/ g->mRules->gamemode
+        != GAME_TYPE_STORY)
+    {
+        if(OptionWhosFirst::WHO_R == options[Options::FIRSTPLAYER].number)
+            initState.player = WRand() % 2;
+        if(OptionWhosFirst::WHO_O == options[Options::FIRSTPLAYER].number)
+            initState.player = 1;
+    }
     g->currentPlayer = g->players[initState.player];
     g->currentActionPlayer = g->currentPlayer;
     g->currentPlayerId = initState.player;
@@ -487,7 +508,7 @@ int Rules::load(string _filename)
     {
         sprintf(filename, JGE_GET_RES("rules/%s").c_str(), _filename.c_str());
     }
-    std::ifstream file(filename);
+    wagic::ifstream file(filename);
     std::string s;
 
     int state = PARSE_UNDEFINED;
