@@ -7,7 +7,11 @@
 #include "TranslateKeys.h"
 #include <hge/hgedistort.h>
 
-//WGuiBase
+/** 
+  Provides an interface to retrieve some standardized colors. The idea here is that a child of WGuiBase 
+  could override, for example, the color of the background based on whether or notit is highlighted 
+  (as WGuiButton does), or be given a particular styling with the WDecoStyled decorator.
+*/
 PIXEL_TYPE WGuiBase::getColor(int type)
 {
     switch (type)
@@ -35,6 +39,10 @@ PIXEL_TYPE WGuiBase::getColor(int type)
     return ARGB(150,50,50,50);
 }
 
+/**
+  Renders the backdrop of a WGui item. 
+  Meant to be overriden in subclasses that require a unique backdrop.
+*/
 void WGuiBase::renderBack(WGuiBase * it)
 {
     if (!it) return;
@@ -44,6 +52,7 @@ void WGuiBase::renderBack(WGuiBase * it)
     else
         subBack(it);
 }
+
 WGuiBase::CONFIRM_TYPE WGuiBase::needsConfirm()
 {
     for (vector<WGuiBase*>::iterator it = items.begin(); it != items.end(); ++it)
@@ -94,7 +103,6 @@ bool WGuiItem::Leaving(JButton key)
 
 void WGuiItem::Render()
 {
-    JRenderer * renderer = JRenderer::GetInstance();
     WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
     DWORD oldcolor = mFont->GetColor();
     mFont->SetColor(getColor(WGuiColor::TEXT));
@@ -190,8 +198,6 @@ void WGuiHeader::Render()
 {
     WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
     mFont->SetColor(getColor(WGuiColor::TEXT));
-
-    JRenderer * renderer = JRenderer::GetInstance();
     mFont->DrawString(_(displayValue).c_str(), x + width / 2, y, JGETEXT_CENTER);
 }
 
@@ -245,13 +251,12 @@ void WGuiMenu::subBack(WGuiBase * item)
 
 }
 
-void WGuiMenu::setSelected(vector<WGuiBase*>::iterator& it)
+void WGuiMenu::setSelected(int newItem)
 {
-    int c = it - items.begin();
-    if (c != currentItem)
+    if (newItem != currentItem)
     {
         items[currentItem]->Leaving(JGE_BTN_NONE);
-        currentItem = c;
+        currentItem = newItem;
         items[currentItem]->Entering(JGE_BTN_NONE);
     }
 }
@@ -441,8 +446,8 @@ void WDecoEnum::Render()
 {
     WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
     mFont->SetColor(getColor(WGuiColor::TEXT));
-    JRenderer * renderer = JRenderer::GetInstance();
     mFont->DrawString(_(getDisplay()).c_str(), getX() + 2, getY() + 3);
+
     OptionInteger* opt = dynamic_cast<OptionInteger*> (it);
     if (opt) mFont->DrawString(_(lookupVal(opt->value)).c_str(), getWidth() - 5, getY() + 3, JGETEXT_RIGHT);
 }
@@ -880,13 +885,37 @@ bool WGuiMenu::CheckUserInput(JButton key)
 {
     bool result = false;
     bool kidModal = false;
-    bool handledInput = false;
     int nbitems = (int) items.size();
     JGE * mEngine = JGE::GetInstance();
+    int i, j;
 
     if (!mEngine->GetButtonState(held)) //Key isn't held down.
     held = JGE_BTN_NONE;
 
+    if (mEngine->GetLeftClickCoordinates(i, j))
+    {   // a dude clicked somwhere, we're gonna select the closest object from where he clicked
+        int n = currentItem;
+        unsigned int distance2;
+        unsigned int minDistance2 = -1;
+        for(size_t k=0; k < items.size(); k++)
+        {
+          WGuiItem* pItem = (WGuiItem*)items[k];
+          distance2 = (pItem-> getY() - j) * (pItem-> getY() - j) + (pItem-> getX() - i) * (pItem-> getX() - i);
+          if (distance2 < minDistance2)
+          {
+              minDistance2 = distance2;
+              n = k;
+          }
+        }
+
+        if (n != currentItem)
+        {
+            setSelected(n);
+            mEngine->LeftClickedProcessed();
+            return true;
+        }
+    }
+
     if (currentItem >= 0 && currentItem < nbitems) kidModal = items[currentItem]->isModal();
 
     if (!kidModal && hasFocus())
@@ -917,37 +946,6 @@ bool WGuiMenu::CheckUserInput(JButton key)
 
     if (currentItem >= 0 && currentItem < nbitems) result = items[currentItem]->CheckUserInput(key);
 
-    if(result == false)
-    { // a dude may have clicked somewhere, we're gonna select the closest object from where he clicked
-      int x, y;
-      unsigned int distance2;
-      unsigned int minDistance2 = -1;
-      int n = currentItem;
-
-      if (mEngine->GetLeftClickCoordinates(x, y))
-      {
-        for(size_t i=0; i < items.size(); i++)
-        {
-          WGuiItem* pItem = (WGuiItem*)items[i];
-          distance2 = (pItem-> getY() - y) * (pItem-> getY() - y) + (pItem-> getX() - x) * (pItem-> getX() - x);
-          if (distance2 < minDistance2)
-          {
-              minDistance2 = distance2;
-              n = i;
-          }
-        }
-
-        if (n != currentItem && items[currentItem] != NULL && items[currentItem]->Leaving(JGE_BTN_DOWN))
-        {
-            currentItem = n;
-            items[currentItem]->Entering(JGE_BTN_DOWN);
-            if (sync) syncMove();
-        }
-        mEngine->LeftClickedProcessed();
-        return true;
-      }
-    }
-
     return result;
 }
 void WGuiMenu::syncMove()
@@ -998,7 +996,6 @@ bool WGuiMenu::isButtonDir(JButton key, int dir)
 void WGuiMenu::Update(float dt)
 {
     int nbitems = (int) items.size();
-    JGE * mEngine = JGE::GetInstance();
 
     if (held) duration += dt;
 
@@ -1116,6 +1113,74 @@ void WGuiTabMenu::Render()
     if (c) c->Render();
 }
 
+bool WGuiTabMenu::CheckUserInput(JButton key)
+{
+    bool result = false;
+    bool kidModal = false;
+    bool handledInput = false;
+    int nbitems = (int) items.size();
+    JGE * mEngine = JGE::GetInstance();
+    int i, j;
+
+    if (!mEngine->GetButtonState(held)) //Key isn't held down.
+        held = JGE_BTN_NONE;
+
+    if (mEngine->GetLeftClickCoordinates(i, j))
+    {
+        if(j <= 25)
+        { // a dude clicked in the tab title bar, let's compute which tab from i
+            float offset = x;
+            WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
+            mFont->SetScale(0.8f);
+            for (vector<WGuiBase*>::iterator it = items.begin(); it != items.end(); it++)
+            {
+                float w = mFont->GetStringWidth(_((*it)->getDisplay()).c_str());
+
+                if(i >= offset+5 && i <= offset+w+10+2)
+                {
+                    setSelected(it);
+                    mEngine->LeftClickedProcessed();
+                    return true;
+                }
+                offset += w + 10 + 2;
+            }
+            mFont->SetScale(1);
+        }
+    }
+
+    if (currentItem >= 0 && currentItem < nbitems) kidModal = items[currentItem]->isModal();
+
+    if (!kidModal && hasFocus())
+    {
+        if (isButtonDir(key, -1))
+        {
+            held = buttonPrev;
+            duration = 0;
+            if (prevItem()) return true;
+        }
+        else if (held == buttonPrev && duration > 1)
+        {
+            duration = .92f;
+            if (prevItem()) return true;
+        }
+        else if (isButtonDir(key, 1))
+        {
+            held = buttonNext;
+            duration = 0;
+            if (nextItem()) return true;
+        }
+        else if (held == buttonNext && duration > 1)
+        {
+            duration = .92f;
+            if (nextItem()) return true;
+        }
+    }
+
+    if (currentItem >= 0 && currentItem < nbitems) result = items[currentItem]->CheckUserInput(key);
+
+    return result;
+}
+
 void WGuiTabMenu::save()
 {
     confirmChange(true);
@@ -1138,10 +1203,10 @@ void WGuiAward::Overlay()
         float fH = mFont->GetHeight();
 
         if (fH < 16) fH = 18;
-        JQuad * button = WResourceManager::Instance()->RetrieveQuad("iconspsp.png", (float) 4 * 32, 0, 32, 32, "", RETRIEVE_NORMAL);
+        JQuadPtr button = WResourceManager::Instance()->RetrieveQuad("iconspsp.png", (float) 4 * 32, 0, 32, 32, "", RETRIEVE_NORMAL);
 
         r->FillRoundRect(5, 10, fW + 32, fH + 2, 2, getColor(WGuiColor::BACK));
-        if (button) r->RenderQuad(button, 10, 12, 0, .5, .5);
+        if (button) r->RenderQuad(button.get(), 10, 12, 0, .5, .5);
         mFont->DrawString(::_(s), 30, 16);
     }
 
@@ -1150,8 +1215,7 @@ void WGuiAward::Overlay()
 void WGuiAward::Underlay()
 {
     char buf[1024];
-    JRenderer * r = JRenderer::GetInstance();
-    JQuad * trophy = NULL;
+    JQuadPtr trophy;
 
     string n = Options::getName(id);
     if (n.size())
@@ -1165,12 +1229,12 @@ void WGuiAward::Underlay()
         trophy = WResourceManager::Instance()->RetrieveTempQuad("trophy_set.png"); //TODO FIXME: Should look in set dir too.
     }
 
-    if (!trophy) //Fallback to basic trophy image.
+    if (!trophy.get()) //Fallback to basic trophy image.
     trophy = WResourceManager::Instance()->RetrieveTempQuad("trophy.png");
 
-    if (trophy)
+    if (trophy.get())
     {
-        r->RenderQuad(trophy, 0, SCREEN_HEIGHT - trophy->mHeight);
+        JRenderer::GetInstance()->RenderQuad(trophy.get(), 0, SCREEN_HEIGHT - trophy->mHeight);
     }
 
 }
@@ -1180,7 +1244,6 @@ void WGuiAward::Render()
 
     if (!goa) return;
 
-    JRenderer * renderer = JRenderer::GetInstance();
     WFont * mFont = WResourceManager::Instance()->GetWFont(Fonts::OPTION_FONT);
     mFont->SetScale(1);
     mFont->SetColor(getColor(WGuiColor::TEXT));
@@ -1191,7 +1254,7 @@ void WGuiAward::Render()
     float fM = fH / 5; //Font Margin is 20% font height
 
     myX += fM;
-    renderer->FillRoundRect(x - fM / 2, y - fM, getWidth() - fM, fH - fM, fM, getColor(WGuiColor::BACK_TAB));
+    JRenderer::GetInstance()->FillRoundRect(x - fM / 2, y - fM, getWidth() - fM, fH - fM, fM, getColor(WGuiColor::BACK_TAB));
     mFont->DrawString(::_(displayValue).c_str(), myX, myY, JGETEXT_LEFT);
 
     myY += fH + 3 * fM;
@@ -1227,7 +1290,7 @@ WGuiAward::~WGuiAward()
 }
 bool WGuiAward::Visible()
 {
-    //WGuiAward is only visible when it's tied to an already acchieved award.
+    //WGuiAward is only visible when it's tied to an already achieved award.
     GameOptionAward * goa = dynamic_cast<GameOptionAward*> (&options[id]);
     if (!goa || !goa->number) return false;
     return true;
@@ -1251,11 +1314,11 @@ void WGuiImage::imageScale(float _w, float _h)
 
 float WGuiImage::getHeight()
 {
-    JQuad * q = NULL;
+    JQuadPtr q =  source->getImage();
 
     if (imgH == 0)
     {
-        if (source && (q = source->getImage())) //Intentional assignment.
+        if (source && q.get())
         return MAX(height,q->mHeight+(2*margin));
     }
 
@@ -1266,15 +1329,14 @@ void WGuiImage::Render()
 {
     if (!source) return;
 
-    JRenderer * renderer = JRenderer::GetInstance();
-    JQuad * q = source->getImage();
+    JQuadPtr q = source->getImage();
     if (q)
     {
         float xS = 1, yS = 1;
         if (imgH != 0 && q->mHeight != 0) yS = imgH / q->mHeight;
         if (imgW != 0 && q->mWidth != 0) xS = imgW / q->mWidth;
 
-        renderer->RenderQuad(q, x + margin, y + margin, 0, xS, yS);
+        JRenderer::GetInstance()->RenderQuad(q.get(), x + margin, y + margin, 0, xS, yS);
     }
 }
 
@@ -1293,7 +1355,7 @@ void WGuiCardImage::Render()
 
     if (!source || (c = source->getCard(mOffset.getPos())) == NULL)
     { //No card, use card back.
-        JQuad * q;
+        JQuadPtr q;
         if (bThumb)
         {
             q = WResourceManager::Instance()->GetQuad("back_thumb");
@@ -1306,13 +1368,13 @@ void WGuiCardImage::Render()
             q = WResourceManager::Instance()->GetQuad("back");
         float scale = p.actZ * 257.f / q->mHeight;
         q->SetColor(ARGB(255,255,255,255));
-        renderer->RenderQuad(q, p.x, p.y, 0, scale, scale);
+        renderer->RenderQuad(q.get(), p.x, p.y, 0, scale, scale);
     }
     else
     { //Have card.
         if (bThumb)
         { //Thumbnail.
-            JQuad * q = NULL;
+            JQuadPtr q;
             if (!options[Options::DISABLECARDS].number)
             {
                 q = source->getThumb(mOffset.getPos());
@@ -1321,14 +1383,19 @@ void WGuiCardImage::Render()
                 q = source->getImage(mOffset.getPos());
 #endif
             }
-            if (!q && (q = CardGui::AlternateThumbQuad(c)) == NULL) return; //TODO Some kind of error image.
-            renderer->RenderQuad(q, p.x, p.y);
+            if (!q.get())
+            {
+                 q = CardGui::AlternateThumbQuad(c);
+                 if (q.get() == NULL)
+                     return; //TODO Some kind of error image.
+            }
+            renderer->RenderQuad(q.get(), p.x, p.y);
         }
         else
         { //Normal card.
-            JQuad * q = source->getImage(mOffset.getPos());
+            JQuadPtr q = source->getImage(mOffset.getPos());
 
-            int mode = (!q || options[Options::DISABLECARDS].number) ? DrawMode::kText : DrawMode::kNormal;
+            int mode = (!q.get() || options[Options::DISABLECARDS].number) ? DrawMode::kText : DrawMode::kNormal;
             CardGui::DrawCard(c, p, mode);
         }
     }
@@ -1348,8 +1415,7 @@ WGuiCardDistort::~WGuiCardDistort()
 
 void WGuiCardDistort::Render()
 {
-    JRenderer * renderer = JRenderer::GetInstance();
-    JQuad * q = NULL;
+    JQuadPtr q;
 
     if (distortSrc)
     {
@@ -1391,7 +1457,7 @@ void WGuiCardDistort::Render()
             if (!q || options[Options::DISABLECARDS].number) q = CardGui::AlternateThumbQuad(c); //TODO alternateX should render to texture.
         }
     }
-    if (!q) return;
+    if (!q.get()) return;
     mesh->SetTexture(q->mTex);
     float x0, y0, w0, h0;
     q->GetTextureRect(&x0, &y0, &w0, &h0);
@@ -1447,10 +1513,6 @@ WDistort::WDistort(float x1, float y1, float x2, float y2, float x3, float y3, f
 
 void WGuiListRow::Render()
 {
-    JRenderer * renderer = JRenderer::GetInstance();
-    int listHeight = 40;
-    int listSelectable = 0;
-    int adjustedCurrent = 0;
     int start = 0, nowPos = 0, vHeight = 0;
     int nbitems = (int) items.size();
 
@@ -1496,7 +1558,6 @@ void WGuiListRow::Render()
 
             items[pos]->setX(x + nowPos);
             items[pos]->setY(y + nowVPos);
-            //items[pos]->setWidth(width/nbitems);
             items[pos]->setWidth(items[pos]->minWidth());
             float temp = items[pos]->getHeight() + 3;
             if (temp > tallestRow) tallestRow = temp;
@@ -1519,6 +1580,7 @@ void WGuiListRow::Render()
     }
     setHeight(tallestRow * numRows + 10);
 }
+
 WGuiListRow::WGuiListRow(string n, WSyncable * s) :
     WGuiList(n, s)
 {
@@ -1527,6 +1589,7 @@ WGuiListRow::WGuiListRow(string n, WSyncable * s) :
     width = SCREEN_WIDTH;
     height = 20;
 }
+
 //WGuiFilterUI
 bool WGuiFilters::Finish(bool emptyset)
 {
@@ -1561,6 +1624,7 @@ bool WGuiFilters::Finish(bool emptyset)
     }
     return true;
 }
+
 void WGuiFilters::ButtonPressed(int controllerId, int controlId)
 {
     if (controllerId == -102)
@@ -1588,6 +1652,7 @@ void WGuiFilters::ButtonPressed(int controllerId, int controlId)
         if (list != NULL) list->ButtonPressed(controllerId, controlId);
     }
 }
+
 void WGuiFilters::buildList()
 {
     list = NEW WGuiList("");
@@ -1601,18 +1666,20 @@ void WGuiFilters::buildList()
     list->Add(wgs);
     list->Entering(JGE_BTN_NONE);
 }
-WGuiFilters::WGuiFilters(string header, WSrcCards * src) :
-    WGuiItem(header)
+
+WGuiFilters::WGuiFilters(string header, WSrcCards * src) : WGuiItem(header)
 {
     bFinished = false;
     source = src;
     recolorTo = -1;
     buildList();
 }
+
 void WGuiFilters::recolorFilter(int color)
 {
     recolorTo = color;
 }
+
 string WGuiFilters::getCode()
 {
     if (!list) return "";
@@ -1642,10 +1709,12 @@ string WGuiFilters::getCode()
     }
     return res;
 }
+
 void WGuiFilters::setSrc(WSrcCards * wsc)
 {
     source = wsc;
 }
+
 void WGuiFilters::Update(float dt)
 {
     if (subMenu && !subMenu->isClosed()) subMenu->Update(dt);
@@ -1667,11 +1736,13 @@ void WGuiFilters::Update(float dt)
         if (bDeleted) wgl->Entering(JGE_BTN_NONE);
     }
 }
+
 void WGuiFilters::Entering(JButton key)
 {
     bFinished = false;
     WGuiItem::Entering(key);
 }
+
 void WGuiFilters::Render()
 {
     if (!list) return; //Hurrah for paranoia.
@@ -1686,6 +1757,7 @@ void WGuiFilters::Render()
 
     if (subMenu && !subMenu->isClosed()) subMenu->Render();
 }
+
 bool WGuiFilters::CheckUserInput(JButton key)
 {
     if (subMenu && !subMenu->isClosed() && subMenu->CheckUserInput(key)) return true;
@@ -1700,11 +1772,13 @@ bool WGuiFilters::CheckUserInput(JButton key)
     }
     return WGuiItem::CheckUserInput(key);
 }
+
 WGuiFilters::~WGuiFilters()
 {
     SAFE_DELETE(list);
     SAFE_DELETE(subMenu);
 }
+
 void WGuiFilters::addColumn()
 {
     if (!list) return;
@@ -1715,6 +1789,7 @@ void WGuiFilters::addColumn()
         wgl->Add(NEW WGuiFilterItem(this));
     }
 }
+
 bool WGuiFilters::isAvailableCode(string code)
 {
     if (!list) return false;
@@ -1733,6 +1808,7 @@ bool WGuiFilters::isAvailableCode(string code)
     }
     return false; //For some reason, we don't have any rows?
 }
+
 bool WGuiFilters::isAvailable(int type)
 {
     if (!list) return false;
@@ -1766,10 +1842,12 @@ bool WGuiFilters::isAvailable(int type)
     }
     return false; //For some reason, we don't have any rows?
 }
+
 void WGuiFilters::clearArgs()
 {
     tempArgs.clear();
 }
+
 void WGuiFilters::addArg(string display, string code)
 {
     if (!subMenu || !isAvailableCode(code)) return;
@@ -1787,7 +1865,7 @@ WGuiFilterItem::WGuiFilterItem(WGuiFilters * parent) :
     mParent = parent;
     mNew = true;
 }
-;
+
 void WGuiFilterItem::updateValue()
 {
     bool delMenu = true;
@@ -2013,6 +2091,7 @@ void WGuiFilterItem::updateValue()
         break;
     }
 }
+
 void WGuiFilterItem::ButtonPressed(int controllerId, int controlId)
 {
     if (!mParent) return;
@@ -2083,6 +2162,7 @@ WGuiKeyBinder::WGuiKeyBinder(string name, GameStateOptions* parent) :
     for (JGE::keybindings_it it = start; it != end; ++it)
         Add(NEW OptionKey(parent, it->first, it->second));
 }
+
 void WGuiKeyBinder::Update(float dt)
 {
     OptionKey* o = dynamic_cast<OptionKey*> (items[0]);
@@ -2096,12 +2176,14 @@ void WGuiKeyBinder::Update(float dt)
         (*it)->Update(dt);
     if (confirmMenu) confirmMenu->Update(dt);
 }
+
 bool WGuiKeyBinder::isModal()
 {
     for (vector<WGuiBase*>::iterator it = items.begin(); it != items.end(); ++it)
         if ((*it)->isModal()) return true;
     return modal;
 }
+
 bool WGuiKeyBinder::CheckUserInput(JButton key)
 {
     if (confirmMenu) return confirmMenu->CheckUserInput(key);
@@ -2109,6 +2191,7 @@ bool WGuiKeyBinder::CheckUserInput(JButton key)
     if (!items[currentItem]->Selectable()) nextItem();
     return true;
 }
+
 void WGuiKeyBinder::setData()
 {
     JGE* j = JGE::GetInstance();
@@ -2205,6 +2288,7 @@ WGuiBase::CONFIRM_TYPE WGuiKeyBinder::needsConfirm()
 
     return CONFIRM_OK;
 }
+
 void WGuiKeyBinder::ButtonPressed(int controllerId, int controlId)
 {
     if (2 == controlId)
@@ -2222,6 +2306,7 @@ void WGuiKeyBinder::ButtonPressed(int controllerId, int controlId)
     SAFE_DELETE(confirmMenu);
     confirmMenu = NULL;
 }
+
 void WGuiKeyBinder::Render()
 {
     WGuiList::Render();
@@ -2245,6 +2330,7 @@ void WGuiKeyBinder::Render()
         confirmMenu->Render();
     }
 }
+
 bool WGuiKeyBinder::yieldFocus()
 {
     return true;
