@@ -5,6 +5,14 @@
 #include "Subtypes.h"
 #include "WResourceManager.h"
 #include "WFont.h"
+#include <sys/stat.h>
+
+namespace wagic
+{
+#ifdef TRACK_FILE_USAGE_STATS
+    std::map<std::string, int> ifstream::sFileMap;
+#endif
+}
 
 using std::vector;
 
@@ -68,16 +76,18 @@ int filesize(const char * filename)
     return file_size;
 }
 
+
+// check to see if a file exists on the file system.
 int fileExists(const char * filename)
 {
-    std::ifstream fichier(filename);
+    wagic::ifstream fichier(filename);
     if (fichier)
     {
         fichier.close();
         return 1;
     }
 
-    std::ifstream fichier2(JGE_GET_RES(filename).c_str());
+    wagic::ifstream fichier2(JGE_GET_RES(filename).c_str());
     if (fichier2)
     {
         fichier2.close();
@@ -86,22 +96,58 @@ int fileExists(const char * filename)
     return 0;
 }
 
+// check to see if a file exists on the file system.
+// this can be used to extract last modified date of a file.
+bool FileExists(const string& strFilename)
+{ 
+    struct stat stFileInfo; 
+    bool blnReturn = false;
+    int intStat; 
+
+    // Attempt to get the file attributes 
+    intStat = stat(strFilename.c_str(),&stFileInfo); 
+    if(intStat == 0)
+    {
+        // We were able to get the file attributes 
+        // so the file obviously exists. 
+        blnReturn = true; 
+    }
+    else
+    {
+        // We were not able to get the file attributes. 
+        // This may mean that we don't have permission to 
+        // access the folder which contains this file. If you 
+        // need to do that level of checking, lookup the 
+        // return values of stat which will give you 
+        // more details on why stat failed. 
+        
+        // try to search in the resource directory 
+        if ( stat( JGE_GET_RES(strFilename).c_str(), &stFileInfo ) == 0)
+            blnReturn = true;
+        else
+            blnReturn = false; 
+    }
+
+    return(blnReturn); 
+}
+
+
 /*
- #ifdef LINUX
+#ifdef LINUX
 
- #include <execinfo.h>
- void dumpStack()
- {
- void* buffer[50];
- int s = backtrace(buffer, 50);
- char** tab = backtrace_symbols(buffer, s);
- for (int i = 1; i < s; ++i) printf("%s\n", tab[i]);
- printf("\n");
- free(tab);
- }
+#include <execinfo.h>
+void dumpStack()
+{
+void* buffer[50];
+int s = backtrace(buffer, 50);
+char** tab = backtrace_symbols(buffer, s);
+for (int i = 1; i < s; ++i) printf("%s\n", tab[i]);
+printf("\n");
+free(tab);
+}
 
- #endif
- */
+#endif
+*/
 
 /* RAM simple check functions source */
 
@@ -190,26 +236,27 @@ u32 ramAvailable(void)
     return size;
 }
 
-string& trim(string &str)
+/* String manipulation functions */
+string& trim(string& str)
 {
     str = ltrim(str);
     str = rtrim(str);
     return str;
 }
 
-string& ltrim(string &str)
+string& ltrim(string& str)
 {
     str.erase(0, str.find_first_not_of(" \t"));
     return str;
 }
 
-string& rtrim(string &str)
+string& rtrim(string& str)
 {
     str.resize(str.find_last_not_of(" \t") + 1);
     return str;
 }
 
-std::vector<std::string> &split(const std::string &s, char delim, std::vector<std::string> &elems)
+std::vector<std::string>& split(const std::string& s, char delim, std::vector<std::string>& elems)
 {
     std::stringstream ss(s);
     std::string item;
@@ -220,19 +267,19 @@ std::vector<std::string> &split(const std::string &s, char delim, std::vector<st
     return elems;
 }
 
-std::string join( vector<string> &v, string delim)
+std::string join(vector<string>& v, string delim)
 {
-	std::string retVal;
-	for ( vector<string>::iterator it = v.begin(); it != v.end(); ++it )
-	{
-		retVal.append( *it );
-		retVal.append( delim );
-	}
+    std::string retVal;
+    for ( vector<string>::iterator it = v.begin(); it != v.end(); ++it )
+    {
+        retVal.append( *it );
+        retVal.append( delim );
+    }
 
-	return retVal;
+    return retVal;
 }
 
-std::vector<std::string> split(const std::string &s, char delim)
+std::vector<std::string> split(const std::string& s, char delim)
 {
     std::vector<std::string> elems;
     return split(s, delim, elems);
@@ -241,52 +288,53 @@ std::vector<std::string> split(const std::string &s, char delim)
 // This is a customized word wrap based on pixel width.  It tries it's best 
 // to wrap strings using spaces as delimiters.  
 // Not sure how this translates into non-english fonts.
-std::string wordWrap(std::string sentence, float width, int fontId)
+std::string wordWrap(const std::string& sentence, float width, int fontId)
 {
-	WFont * mFont = WResourceManager::Instance()->GetWFont(fontId);
-	float lineWidth = mFont->GetStringWidth( sentence.c_str() );
-	string retVal = sentence;
-	if ( lineWidth < width ) return sentence;
-   
-	int numLines = 1;
-	int breakIdx = 0;
-	for( size_t idx = 0; idx < sentence.length(); idx ++ )
+    WFont * mFont = WResourceManager::Instance()->GetWFont(fontId);
+    float lineWidth = mFont->GetStringWidth( sentence.c_str() );
+    string retVal = sentence;
+    if ( lineWidth < width ) return sentence;
+
+    int numLines = 1;
+    int breakIdx = 0;
+    for( size_t idx = 0; idx < sentence.length(); idx ++ )
     {
-		if ( sentence[idx] == ' ' )
-		{
-			string currentSentence = sentence.substr(breakIdx, idx - breakIdx);
-			float stringLength = mFont->GetStringWidth( currentSentence.c_str() );
-			if (stringLength >= width)
-			{				
-				if ( stringLength > width )
-				{
-					while ( sentence[idx-1] != ' ' )
-						idx--;
-				}
-				retVal[idx-1] = '\n';				
-				breakIdx = idx;
-				numLines++;
-			}
-		}
-		else if ( sentence[idx] == '\n' )
-		{
-			string currentSentence = sentence.substr(breakIdx, idx - breakIdx);
-			float stringLength = mFont->GetStringWidth( currentSentence.c_str() );
-			if (stringLength >= width)
-			{				
-				if ( stringLength > width )
-				{
-					while ( sentence[idx-1] != ' ' )
-						idx--;
-					retVal[idx-1] = '\n';				
-				}
-				numLines++;
-			}
-			breakIdx = idx;
-			numLines++;
-		}
+        if ( sentence[idx] == ' ' )
+        {
+            string currentSentence = sentence.substr(breakIdx, idx - breakIdx);
+            float stringLength = mFont->GetStringWidth( currentSentence.c_str() );
+            if (stringLength >= width)
+            {				
+                if ( stringLength > width )
+                {
+                    while ( sentence[idx-1] != ' ' )
+                        idx--;
+                }
+                retVal[idx-1] = '\n';				
+                breakIdx = idx;
+                numLines++;
+            }
+        }
+        else if ( sentence[idx] == '\n' )
+        {
+            string currentSentence = sentence.substr(breakIdx, idx - breakIdx);
+            float stringLength = mFont->GetStringWidth( currentSentence.c_str() );
+            if (stringLength >= width)
+            {				
+                if ( stringLength > width )
+                {
+                    while ( sentence[idx-1] != ' ' )
+                        idx--;
+                    retVal[idx-1] = '\n';				
+                }
+                numLines++;
+            }
+            breakIdx = idx;
+            numLines++;
+        }
     }
 
     return retVal;
 }
 
+
